@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,14 +19,33 @@
  *
  */
 
+/**
+ * @defgroup SmlObjectAuthPrivate SyncML Authentication Object Internals
+ * @ingroup PrivateObjectAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_AUTH_INTERNALS_H_
 #define _SML_AUTH_INTERNALS_H_
 
+#include "sml_auth.h"
+
 struct SmlAuthenticator {
 	SmlErrorType state;
 	SmlAuthVerifyCb verifyCallback;
 	void *verifyCallbackUserdata;
 	SmlBool enabled;
+	SmlAuthType type;
 };
 
+char *smlAuthGetCredString(
+	SmlAuthType type,
+	const char *username,
+	const char *password,
+	const char *b64_nonce,
+	SmlError **error);
+
 #endif //_SML_AUTH_INTERNALS_H_
+/*@}*/
