@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,16 +27,79 @@
 #include <libsyncml/sml_session_internals.h>
 #include <libsyncml/sml_elements_internals.h>
 #include <libsyncml/sml_command_internals.h>
+#include "libsyncml/sml_error_internals.h"
 
+#ifdef WIN32
+#include <windef.h>
+#else
 #include<sys/utsname.h>
+#endif
 
-/**
- * @defgroup GroupIDPrivate Group Description Internals
- * @ingroup ParentGroupID
- * @brief The private part
+/* Design notice
  * 
+ * There is only one SmlDevInfAgent per SmlManager
+ * because there is only one local device information.
+ * Therefore the agent must be able to manage one
+ * device information per session because every remote
+ * device can have different capabilities.
  */
-/*@{*/
+
+/* functions to manage SmlDevInfAgentSession */
+
+static SmlDevInfAgentSession* _new_session(
+				SmlDevInfAgent *agent,
+				SmlSession *session,
+				SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, agent, session, error);
+
+	CHECK_ERROR_REF
+	smlAssert(agent);
+	smlAssert(agent->sessions);
+	smlAssert(session);
+
+	SmlDevInfAgentSession *as = smlTryMalloc0(sizeof(SmlDevInfAgentSession), error);
+	if (!as)
+		goto error;
+
+	g_hash_table_insert(agent->sessions, session, as);
+
+	smlTrace(TRACE_EXIT, "%s - %p", __func__, as);
+	return as;
+error:
+	smlSafeFree((gpointer *) &as);
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+	return NULL;
+}
+
+static SmlDevInfAgentSession* _get_session(
+				SmlDevInfAgent *agent,
+				SmlSession *session)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, agent, session);
+
+	smlAssert(agent);
+	smlAssert(agent->sessions);
+	smlAssert(session);
+
+	SmlDevInfAgentSession *as = g_hash_table_lookup(agent->sessions, session);
+	smlTrace(TRACE_EXIT, "%s - %p", __func__, as);
+	return as;
+}
+
+static void _free_session (gpointer data)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, data);
+
+	SmlDevInfAgentSession *as = data;
+	if (as->recvDevInf)
+		smlDevInfUnref(as->recvDevInf);
+	smlSafeFree((gpointer *) &as);
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+}
+
+/* functions to manage SmlDevInfAgent */
 
 static void _get_devinf_reply(SmlSession *session, SmlStatus *status, void *userdata)
 {
@@ -46,9 +110,22 @@ static void _get_devinf_reply(SmlSession *session, SmlStatus *status, void *user
 	if (smlStatusIsResult(status)) {
 		SmlCommand *result = smlStatusGetResult(status);
 		
-		agent->recvDevInf = smlDevInfFromResult(result, &error);
-		if (!agent->recvDevInf)
+		/* get cached session or add session to agent cache */
+
+		SmlDevInfAgentSession *as = _get_session(agent, session);
+		if (!as) {
+			as = _new_session(agent, session, &error);
+			if (!as)
+				goto error;
+		}
+
+		/* cache device information */
+
+		as->recvDevInf = smlDevInfFromResult(result, &error);
+		if (!as->recvDevInf)
 			goto error;
+
+		/* send answer */
 		
 		SmlStatus *reply = smlCommandNewReply(result, SML_NO_ERROR, &error);
 		if (!reply)
@@ -78,24 +155,60 @@ static void _devinf_reply(SmlSession *session, SmlStatus *status, void *userdata
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-/*@}*/
-
-/**
- * @defgroup GroupID Group Description
- * @ingroup ParentGroupID
- * @brief What does this group do?
- * 
- */
-/*@{*/
-
-
 static SmlBool _send_devinf(SmlDevInfAgent *agent, SmlSession *session, SmlCommand *get, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, agent, session, get, error);
+	CHECK_ERROR_REF
 	SmlCommand *result = NULL;
 	SmlCommand *cmd = NULL;
-	
-	if (!agent->devinfSent) {
+
+	/* It is a good idea to perform some sanity checks on the
+	 * configuration before the device information is sent.
+	 */
+
+	/* If this is SyncML 1.1 or higher then large object support is
+	 * required for OMA DS servers.
+	 */
+	if (smlSessionGetVersion(session) >= SML_VERSION_11 &&
+	    session->sessionType == SML_SESSION_TYPE_SERVER &&
+	    !smlDevInfSupportsLargeObjs(agent->devinf))
+	{
+		smlErrorSet(error, SML_ERROR_INTERNAL_MISCONFIGURATION,
+			"OMA DS Server must support large object handling if SyncML 1.1 or higher is used.");
+		goto error;
+	}
+
+	/* If large object support is enabled then MaxMsgSize and
+	 * MaxObjSize must be set.
+	 */
+	if (smlDevInfSupportsLargeObjs(agent->devinf))
+	{
+		if (smlSessionGetLocalMaxMsgSize(session) < 1)
+		{
+			smlErrorSet(error, SML_ERROR_INTERNAL_MISCONFIGURATION,
+				"If large object support is enabled then MaxMsgSize must be set.");
+			goto error;
+		}
+		if (smlSessionGetLocalMaxObjSize(session) < 1)
+		{
+			smlErrorSet(error, SML_ERROR_INTERNAL_MISCONFIGURATION,
+				"If large object support is enabled then MaxObjSize must be set.");
+			goto error;
+		}
+	}
+
+	/* get cached session or add session to agent cache */
+
+	SmlDevInfAgentSession *as = _get_session(agent, session);
+	if (!as) {
+		as = _new_session(agent, session, error);
+		if (!as)
+			goto error;
+	}
+
+	/* Sanity checks complete. Sending device information ... */
+
+	if (!as->devinfSent) {
 		if (get) {
 			if (smlSessionGetVersion(session) == SML_VERSION_10)
 				result = smlDevInfNewResult(get, agent->devinf, SML_DEVINF_VERSION_10, error);
@@ -142,22 +255,36 @@ static SmlBool _send_devinf(SmlDevInfAgent *agent, SmlSession *session, SmlComma
 
 			smlCommandUnref(cmd);
 		}
-		agent->devinfSent = TRUE;
+		as->devinfSent = TRUE;
 	} else {
-		smlTrace(TRACE_INTERNAL, "Already sent the devinf!");
-	
-		/* We return an generic error if we dont want to send the devinf to a get
-		 * request. Hope that all devices like this */
-		SmlStatus *reply = smlCommandNewReply(get, SML_ERROR_GENERIC, error);
-		if (!reply)
-			goto error;
-		
-		if (!smlSessionSendReply(session, reply, error)) {
+		smlTrace(TRACE_INTERNAL, "%s: Already sent the devinf!", __func__);
+
+		if (get) {
+			/* We return a generic error if we dont want to send the devinf to a get
+			 * request. Hope that all devices like this */
+			SmlStatus *reply = smlCommandNewReply(get, SML_ERROR_GENERIC, error);
+			if (!reply)
+				goto error;
+
+			if (!smlSessionSendReply(session, reply, error)) {
+				smlStatusUnref(reply);
+				goto error;
+			}
+
 			smlStatusUnref(reply);
-			goto error;
+		} else {
+			/* This means that a software component tries to
+			 * send the device information but it was already
+			 * sent. This can happen if the local library
+			 * user is missing the remote device information
+			 * and wants to send the get request together
+			 * with its own device information. Nevertheless
+			 * this is an error - but it can be ignored.
+			 */
+			smlTrace(TRACE_INTERNAL,
+				"%s: libsyncml does not send local device information twice.",
+				__func__);
 		}
-		
-		smlStatusUnref(reply);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -179,8 +306,15 @@ static void _recv_devinf(SmlSession *session, SmlCommand *cmd, void *userdata)
 	if (!smlItemGetData(cmd->private.access.item, &data, &size, &error))
 		goto error;
 
-	agent->recvDevInf = smlDevInfParse(data, size, &error);
-	if (!agent->recvDevInf)
+	SmlDevInfAgentSession *as = _get_session(agent, session);
+	if (!as) {
+		as = _new_session(agent, session, &error);
+		if (!as)
+			goto error;
+	}
+
+	as->recvDevInf = smlDevInfParse(data, size, &error);
+	if (!as->recvDevInf)
 		goto error;
 
 	SmlStatus *reply = smlCommandNewReply(cmd, SML_NO_ERROR, &error);
@@ -198,6 +332,7 @@ static void _recv_devinf(SmlSession *session, SmlCommand *cmd, void *userdata)
 	return;
 	
 error:
+	smlSessionDispatchEvent(session, SML_SESSION_EVENT_ERROR, NULL, NULL, NULL, error);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(&error));
 	smlErrorDeref(&error);
 }
@@ -221,15 +356,19 @@ error:
 SmlDevInfAgent *smlDevInfAgentNew(SmlDevInf *devinf, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, devinf, error);
+	CHECK_ERROR_REF
 	smlAssert(devinf);
 	
 	SmlDevInfAgent *agent = smlTryMalloc0(sizeof(SmlDevInfAgent), error);
 	if (!agent)
 		goto error;
 
-	agent->devinfSent = FALSE;
 	agent->devinf = devinf;
-	agent->recvDevInf = NULL;
+	agent->sessions = g_hash_table_new_full(NULL, NULL, NULL, _free_session);
+	if (!agent->sessions) {
+		smlErrorSet(error, SML_ERROR_INTERNAL_NO_MEMORY, "Cannot create new hash table.");
+		goto error;
+	}
 
 	if (!smlDevInfGetManufacturer(devinf))
 		smlDevInfSetManufacturer(devinf, "OpenSync");
@@ -237,21 +376,36 @@ SmlDevInfAgent *smlDevInfAgentNew(SmlDevInf *devinf, SmlError **error)
 		smlDevInfSetModel(devinf, "libsyncml");
 	if (!smlDevInfGetOEM(devinf))
 	{
+#ifdef WIN32
+		smlDevInfSetOEM(devinf, "Windows");
+		DWORD dwVersion = GetVersion();
+		DWORD dwMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
+		DWORD dwMinorVersion = (DWORD)(HIBYTE(LOWORD(dwVersion))); 
+		DWORD dwBuild = 0;
+		if (dwVersion < 0x80000000) dwBuild = (DWORD)(HIWORD(dwVersion));
+		
+		char szVersion[1024];
+		sprintf(szVersion, "%d.%d (%d)", dwMajorVersion, dwMinorVersion, dwBuild);
+		smlDevInfSetFirmwareVersion(devinf, szVersion);
+#else
 		struct utsname *buf = malloc(sizeof(struct utsname));
 		if (uname(buf) == 0)
 		{
 			smlDevInfSetOEM(devinf, buf->sysname);
 			smlDevInfSetFirmwareVersion(devinf, buf->release);
 		}
-		free(buf);
+		smlSafeFree((gpointer *)&buf);
+#endif
 	}
 	if (!smlDevInfGetSoftwareVersion(devinf))
 		smlDevInfSetSoftwareVersion(devinf, VERSION);
-	
+
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, agent);
 	return agent;
 
 error:
+	if (agent)
+		smlSafeFree((gpointer *) &agent);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
 }
@@ -262,30 +416,85 @@ void smlDevInfAgentFree(SmlDevInfAgent *agent)
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, agent);
 	smlAssert(agent);
 	
-	g_free(agent);
+	if (agent->devinf)
+		smlDevInfUnref(agent->devinf);
+	g_hash_table_destroy(agent->sessions);
+	smlSafeFree((gpointer *)&agent);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);	
 }
 
-/* Set the devinf of the remote peer. */
+/* FIXME: DEPRECATED */
 void smlDevInfAgentSetDevInf(SmlDevInfAgent *agent, SmlDevInf *devinf)
 {
+	smlTrace(TRACE_ERROR, "%s(%p, %p)", __func__, agent, devinf);
+	smlAssertMsg(NULL, "This function is a design bug.");
+}
+
+/* FIXME: DEPRECATED */
+SmlDevInf *smlDevInfAgentGetDevInf(SmlDevInfAgent *agent)
+{
+	smlTrace(TRACE_ERROR, "%s(%p, %p)", __func__, agent);
+	smlAssertMsg(NULL, "This function is a design bug.");
+	return NULL;
+}
+
+/* Set the devinf of the remote peer. */
+SmlBool smlDevInfAgentSetSessionDevInf(
+		SmlDevInfAgent *agent,
+		SmlSession *session,
+		SmlDevInf *devinf,
+		SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, agent, session, devinf, error);
+
+	CHECK_ERROR_REF
 	smlAssert(agent);
+	smlAssert(agent->sessions);
+	smlAssert(session);
 	smlAssert(devinf);
-	agent->recvDevInf = devinf;
+
+	SmlDevInfAgentSession *as = _get_session(agent, session);
+	if (!as) {
+		as = _new_session(agent, session, error);
+		if (!as)
+			goto error;
+	}
+
+	as->recvDevInf = devinf;
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+	return FALSE;
 }
 
 /* Get the devinf that was sent in the session. Returns FALSE if no devinf was received yet. */
-SmlDevInf *smlDevInfAgentGetDevInf(SmlDevInfAgent *agent)
+SmlDevInf *smlDevInfAgentGetSessionDevInf(SmlDevInfAgent *agent, SmlSession *session)
 {
+	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, agent, session);
+
 	smlAssert(agent);
-	return agent->recvDevInf;
+	smlAssert(agent->sessions);
+	smlAssert(session);
+
+	SmlDevInfAgentSession *as = _get_session(agent, session);
+	if (!as)
+	{
+		smlTrace(TRACE_EXIT, "%s - the session is not cached until now", __func__);
+		return NULL;
+	}
+
+	smlTrace(TRACE_EXIT, "%s - %p", __func__, as->recvDevInf);
+	return as->recvDevInf;
 }
 
 /** Issues a put request on the session if needed */
 SmlBool smlDevInfAgentSendDevInf(SmlDevInfAgent *agent, SmlSession *session, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, agent, session, error);
+	CHECK_ERROR_REF
 	smlAssert(agent);
 	
 	if (!_send_devinf(agent, session, NULL, error))
@@ -303,30 +512,45 @@ error:
 SmlBool smlDevInfAgentRequestDevInf(SmlDevInfAgent *agent, SmlSession *session, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, agent, session, error);
+	CHECK_ERROR_REF
 	smlAssert(agent);
 	SmlCommand *get = NULL;
+
+	SmlDevInfAgentSession *as = _get_session(agent, session);
+	if (!as) {
+		as = _new_session(agent, session, error);
+		if (!as)
+			goto error;
+	}
 	
-	if (agent->recvDevInf) {
+	if (as->recvDevInf) {
 		smlTrace(TRACE_EXIT, "%s: Already have the devinf", __func__);
 		return TRUE;
 	}
 	
-	if (smlSessionGetVersion(session) == SML_VERSION_10)
-		get = smlDevInfNewGet(SML_DEVINF_VERSION_10, error);
-	else if (smlSessionGetVersion(session) == SML_VERSION_12)
-		get = smlDevInfNewGet(SML_DEVINF_VERSION_12, error);
-	else
-		get = smlDevInfNewGet(SML_DEVINF_VERSION_11, error);
-	
-	if (!get)
-		goto error;
-		
-	if (!smlSessionSendCommand(session, get, NULL, _get_devinf_reply, agent, error)) {
+	if (!as->devinfRequested)
+	{
+		if (smlSessionGetVersion(session) == SML_VERSION_10)
+			get = smlDevInfNewGet(SML_DEVINF_VERSION_10, error);
+		else if (smlSessionGetVersion(session) == SML_VERSION_12)
+			get = smlDevInfNewGet(SML_DEVINF_VERSION_12, error);
+		else
+			get = smlDevInfNewGet(SML_DEVINF_VERSION_11, error);
+
+		if (!get)
+			goto error;
+
+		if (!smlSessionSendCommand(session, get, NULL, _get_devinf_reply, agent, error)) {
+			smlCommandUnref(get);
+			goto error;
+		}
+
 		smlCommandUnref(get);
-		goto error;
+
+		as->devinfRequested = TRUE;
+	} else {
+		smlTrace(TRACE_INTERNAL, "%s: Already requested the devinf!", __func__);
 	}
-	
-	smlCommandUnref(get);
 			
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -339,6 +563,7 @@ error:
 SmlBool smlDevInfAgentRegisterSession(SmlDevInfAgent *agent, SmlManager *manager, SmlSession *session, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, agent, manager, session, error);
+	CHECK_ERROR_REF
 	smlAssert(agent);
 	smlAssert(manager);
 
@@ -370,7 +595,7 @@ SmlBool smlDevInfAgentRegisterSession(SmlDevInfAgent *agent, SmlManager *manager
 	 * the device information of a server.
 	 */
 
-	smlTrace(TRACE_INTERNAL, "register callbacks for PUT command");
+	smlTrace(TRACE_INTERNAL, "%s: register callbacks for PUT command", __func__);
 	if (!smlManagerObjectRegister(
 			manager, SML_COMMAND_TYPE_PUT, session, 
 			NULL, devinf10, NULL, _recv_devinf, NULL, agent, 
@@ -407,7 +632,7 @@ SmlBool smlDevInfAgentRegisterSession(SmlDevInfAgent *agent, SmlManager *manager
 	 * device information.
 	 */
 
-	smlTrace(TRACE_INTERNAL, "register callbacks for GET command");
+	smlTrace(TRACE_INTERNAL, "%s: register callbacks for GET command", __func__);
 	if (!smlManagerObjectRegister(
 			manager, SML_COMMAND_TYPE_GET, session, 
 			devinf10, NULL, NULL, _request_devinf, NULL, agent, 
@@ -440,7 +665,7 @@ SmlBool smlDevInfAgentRegisterSession(SmlDevInfAgent *agent, SmlManager *manager
 	 * the device information of a server.
 	 */
 
-	smlTrace(TRACE_INTERNAL, "register callbacks for RESULTS command");
+	smlTrace(TRACE_INTERNAL, "%s: register callbacks for RESULTS command", __func__);
 	if (!smlManagerObjectRegister(
 			manager, SML_COMMAND_TYPE_RESULTS, session, 
 			devinf10, NULL, NULL, _recv_devinf, NULL, agent, 
@@ -479,6 +704,7 @@ error:
 SmlBool smlDevInfAgentRegister(SmlDevInfAgent *agent, SmlManager *manager, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, agent, manager, error);
+	CHECK_ERROR_REF
 
 	SmlBool retval = smlDevInfAgentRegisterSession(agent, manager, NULL, error);
 	
@@ -493,4 +719,3 @@ error:
 	return FALSE;
 }
 
-/*@}*/
