@@ -4,8 +4,8 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,14 +18,30 @@
  *
  */
 
+/**
+ * @defgroup SmlObjectDevInfPrivate SyncML Device Information Object Internals
+ * @ingroup PrivateObjectAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_DEVINF_OBJ_INTERNALS_H_
 #define _SML_DEVINF_OBJ_INTERNALS_H_
 
+typedef struct SmlDevInfAgentSession {
+	/* states */
+	SmlBool devinfSent;
+	SmlBool devinfRequested;
+	/* remote device information */
+	SmlDevInf *recvDevInf;
+	
+} SmlDevInfAgentSession;
+
 struct SmlDevInfAgent {
 	SmlDevInf *devinf;
-	SmlBool devinfSent;
-	
-	SmlDevInf *recvDevInf;
+	GHashTable *sessions;
 };
 
 #endif //_SML_DEVINF_OBJ_INTERNALS_H_
+/*@}*/
