@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,30 +24,27 @@
 
 #include <libsyncml/syncml_internals.h>
 #include "sml_ds_server_internals.h"
+#include <libsyncml/sml_error_internals.h>
+#include <libsyncml/sml_manager_internals.h>
 #include <libsyncml/sml_session_internals.h>
 #include <libsyncml/sml_command_internals.h>
 #include <libsyncml/sml_elements_internals.h>
 
-/**
- * @defgroup GroupIDPrivate Group Description Internals
- * @ingroup ParentGroupID
- * @brief The private part
- * 
- */
-/*@{*/
-
 static SmlWriteContext *_write_context_find(SmlDsSession *dsession, const char *uid, SmlChangeType type)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %s, %i)", __func__, dsession, uid, type);
+	smlTrace(TRACE_ENTRY, "%s(%p, %s, %i)", __func__, dsession, VA_STRING(uid), type);
 	
+	g_mutex_lock(dsession->pendingMapsLock);
 	GList *c = NULL;
 	for (c = dsession->pendingMaps; c; c = c->next) {
 		SmlWriteContext *ctx = c->data;
 		if (!strcmp(uid, ctx->uid) && ctx->type == type) {
+			g_mutex_unlock(dsession->pendingMapsLock);
 			smlTrace(TRACE_EXIT, "%s: %p", __func__, ctx);
 			return ctx;
 		}
 	}
+	g_mutex_unlock(dsession->pendingMapsLock);
 	
 	smlTrace(TRACE_EXIT_ERROR, "%s: Not found", __func__);
 	return NULL;
@@ -60,12 +58,12 @@ static void _write_context_free(SmlWriteContext *ctx)
 		smlStatusUnref(ctx->status);
 	
 	if (ctx->uid)
-		g_free(ctx->uid);
+		smlSafeCFree(&(ctx->uid));
 		
 	if (ctx->newuid)
-		g_free(ctx->newuid);
+		smlSafeCFree(&(ctx->newuid));
 	
-	g_free(ctx);
+	smlSafeFree((gpointer *)&ctx);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -79,30 +77,41 @@ static void _write_context_dispatch(SmlDsSession *dsession, SmlWriteContext *ctx
 		return;
 	}
 	
-	smlTrace(TRACE_INTERNAL, "Dispatching: uid %s, Type %i, newuid %s, result %i", ctx->uid, ctx->type, ctx->newuid, smlStatusGetCode(ctx->status));
+	smlTrace(TRACE_INTERNAL, "%s: Dispatching: uid %s, Type %i, newuid %s, result %i", __func__, VA_STRING(ctx->uid), ctx->type, VA_STRING(ctx->newuid), smlStatusGetCode(ctx->status));
 	
-	if (((ctx->type != SML_CHANGE_ADD || smlStatusGetClass(ctx->status) != SML_ERRORCLASS_SUCCESS) && !ctx->newuid) || dsession->server->servertype == SML_DS_CLIENT) {
-		ctx->callback(dsession, ctx->status, NULL, ctx->userdata);
-		
-		_write_context_free(ctx);
-		
-		dsession->pendingMaps = g_list_remove(dsession->pendingMaps, ctx);
-		smlTrace(TRACE_EXIT, "%s", __func__);
-		return;
-	}
-	
-	if (!ctx->newuid) {
+	if (ctx->type == SML_CHANGE_ADD &&
+	    smlStatusGetClass(ctx->status) == SML_ERRORCLASS_SUCCESS &&
+	    !dsession->recvMappingCallback &&
+	    !ctx->newuid &&
+	    dsession->server->servertype == SML_DS_SERVER)
+	{
 		smlTrace(TRACE_EXIT, "%s: No mapitem yet", __func__);
 		return;
 	}
-	
+
 	ctx->callback(dsession, ctx->status, ctx->newuid, ctx->userdata);
 	
+	g_mutex_lock(dsession->pendingMapsLock);
+	dsession->pendingMaps = g_list_remove(dsession->pendingMaps, ctx);
+	g_mutex_unlock(dsession->pendingMapsLock);
+
+	if (ctx->type == SML_CHANGE_ADD &&
+	    smlStatusGetClass(ctx->status) == SML_ERRORCLASS_SUCCESS &&
+	    dsession->recvEventCallback &&
+	    !dsession->pendingMaps)
+	{
+		/* FIXME: I'm sure that this event is buggy.
+		 * FIXME: What is the semantic of this event?
+		 * Now there are no more MapItems ...
+		 * do some fancy callback. Some applications will love it ;)
+		 */
+		dsession->recvEventCallback(dsession, SML_DS_EVENT_COMMITEDCHANGES, dsession->recvEventCallbackUserdata);
+		smlTrace(TRACE_INTERNAL, "%s: recvEventCallback commited changes callback", __func__);
+	}
+
 	_write_context_free(ctx);
 		
-	dsession->pendingMaps = g_list_remove(dsession->pendingMaps, ctx);
-
-	smlTrace(TRACE_EXIT, "%s: Dispatched add", __func__);
+	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
 static void _alert_reply(SmlSession *session, SmlStatus *status, void *userdata)
@@ -129,6 +138,12 @@ static void _sync_reply(SmlSession *session, SmlStatus *status, void *userdata)
 	
 	dsession->sentSyncCallback = NULL;
 	dsession->sentSyncCallbackUserdata = NULL;
+
+	// Now there are no more MapItems ... do some fancy callback. Some applications will love it ;)
+	if (dsession->recvEventCallback && !dsession->pendingMaps) {
+		dsession->recvEventCallback(dsession, SML_DS_EVENT_COMMITEDCHANGES, dsession->recvEventCallbackUserdata);
+		smlTrace(TRACE_INTERNAL, "%s: recvEventCallback commited changes callback", __func__);
+	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -141,14 +156,28 @@ static void _change_reply(SmlSession *session, SmlStatus *status, void *userdata
 	SmlWriteContext *ctx = userdata;
 	SmlDsSession *dsession = ctx->session;
 	
-	if (status->type == SML_COMMAND_TYPE_ADD) {
+	/* Please see smlChangeAssemble in sml_xml_parser.c for
+	 * a more detailed description on the checked behaviour.
+	 * If the received command is an Add or the remote peer
+	 * is an OMA DS client then the remote peer must send a
+	 * Source. Otherwise a Target must be sent by the remote
+	 * peer. We only check the required behaviour and not
+	 * the optional one.
+	 */
+	if (status->type == SML_COMMAND_TYPE_ADD ||
+	    session->sessionType == SML_SESSION_TYPE_CLIENT) {
+		/* FIXME: How do we handle remote peers which are clients
+		 * FIXME: and only support replace? The assembler will
+		 * FIXME: send a replace command with a target instead of
+		 * FIXME: an add command with a source.
+		 */
 		if (!status->sourceRef) {
-			smlTrace(TRACE_EXIT_ERROR, "%s: Received add status without sourceRef", __func__);
+			smlTrace(TRACE_EXIT_ERROR, "%s: Received add status or change reply from an OMA DS server without sourceRef", __func__);
 			return;
 		}
-	} else if (status->type == SML_COMMAND_TYPE_REPLACE || status->type == SML_COMMAND_TYPE_DELETE) {
+	} else {
 		if (!status->targetRef) {
-			smlTrace(TRACE_EXIT_ERROR, "%s: Received delete or modify status without targetRef", __func__);
+			smlTrace(TRACE_EXIT_ERROR, "%s: Received delete or modify status or change reply from an OMA DS client without targetRef", __func__);
 			return;
 		}
 	}
@@ -160,20 +189,11 @@ static void _change_reply(SmlSession *session, SmlStatus *status, void *userdata
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-/*@}*/
-
-/**
- * @defgroup GroupID Group Description
- * @ingroup ParentGroupID
- * @brief What does this group do?
- * 
- */
-/*@{*/
-
 SmlDsServer *smlDsServerNew(const char *type, SmlLocation *location, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %p, %p)", __func__, type, location, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %p, %p)", __func__, VA_STRING(type), location, error);
 	smlAssert(location);
+	CHECK_ERROR_REF
 	
 	SmlDsServer *server = smlTryMalloc0(sizeof(SmlDsServer), error);
 	if (!server)
@@ -195,9 +215,10 @@ error:
 
 SmlDsServer *smlDsClientNew(const char *type, SmlLocation *location, SmlLocation *target, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %p, %p, %p)", __func__, type, location, target, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %p, %p, %p)", __func__, VA_STRING(type), location, target, error);
 	smlAssert(location);
 	smlAssert(target);
+	CHECK_ERROR_REF
 	
 	SmlDsServer *server = smlTryMalloc0(sizeof(SmlDsServer), error);
 	if (!server)
@@ -238,9 +259,9 @@ void smlDsServerFree(SmlDsServer *server)
 		smlLocationUnref(server->target);
 	
 	if (server->contenttype)
-		g_free(server->contenttype);
+		smlSafeCFree(&(server->contenttype));
 	
-	g_free(server);
+	smlSafeFree((gpointer *)&server);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);	
 }
@@ -278,6 +299,18 @@ void smlDsServerSetSanCallback(SmlDsServer *server, SmlDsServerSanCb callback, v
 	smlTrace(TRACE_EXIT, "%s", __func__);	
 }
 
+void smlDsServerSetSanSessionCallback(SmlDsServer *server, SmlDsServerSanSessionCb callback, void *userdata)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, server, callback, userdata);
+	smlAssert(server);
+	smlAssert(server->servertype == SML_DS_CLIENT);
+	
+	server->sanSessionCallback = callback;
+	server->sanSessionCallbackUserdata = userdata;
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);	
+}
+
 const char *smlDsServerGetLocation(SmlDsServer *server)
 {
 	smlAssert(server);
@@ -297,6 +330,7 @@ SmlBool smlDsServerAddSan(SmlDsServer *server, SmlNotification *san, SmlError **
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, server, san, error);
 	smlAssert(server);
 	smlAssert(san);
+	CHECK_ERROR_REF
 	
 	if (!smlNotificationNewAlert(san, SML_ALERT_TWO_WAY_BY_SERVER, server->contenttype, smlLocationGetURI(server->location), error))
 		goto error;
@@ -336,9 +370,10 @@ error:
 
 SmlDsSession *smlDsServerSendAlert(SmlDsServer *server, SmlSession *session, SmlAlertType type, const char *last, const char *next, SmlStatusReplyCb callback, void *userdata, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %s, %s, %p)", __func__, server, session, type, last, next, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %s, %s, %p)", __func__, server, session, type, VA_STRING(last), VA_STRING(next), error);
 	smlAssert(server);
 	smlAssert(session);
+	CHECK_ERROR_REF
 	
 	SmlDsSession *dsession = smlDsSessionNew(server, session, error);
 	if (!dsession)
@@ -362,7 +397,7 @@ SmlDsSession *smlDsServerSendAlert(SmlDsServer *server, SmlSession *session, Sml
 	return dsession;
 
 error_free_dsession:
-	g_free(dsession);
+	smlSafeFree((gpointer *)&dsession);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
@@ -372,6 +407,7 @@ SmlDsSession *smlDsSessionNew(SmlDsServer *server, SmlSession *session, SmlError
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, server, session, error);
 	smlAssert(server);
+	CHECK_ERROR_REF
 	
 	SmlDsSession *dsession = smlTryMalloc0(sizeof(SmlDsSession), error);
 	if (!dsession)
@@ -379,6 +415,7 @@ SmlDsSession *smlDsSessionNew(SmlDsServer *server, SmlSession *session, SmlError
 
 	dsession->server = server;
 	dsession->session = session;
+	dsession->write_lock = g_mutex_new();
 	dsession->lock = g_mutex_new();
 	dsession->syncReply = SML_ERROR_UNKNOWN;
 	dsession->refCount = 1;
@@ -390,6 +427,8 @@ SmlDsSession *smlDsSessionNew(SmlDsServer *server, SmlSession *session, SmlError
 	
 	dsession->location = server->location;
 	smlLocationRef(dsession->location);
+
+	dsession->pendingMapsLock = g_mutex_new();
 	
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, dsession);
 	return dsession;
@@ -416,7 +455,7 @@ void smlDsSessionUnref(SmlDsSession *dsession)
 	smlAssert(dsession);
 	
 	if (g_atomic_int_dec_and_test(&(dsession->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (dsession->target)
 			smlLocationUnref(dsession->target);
@@ -442,12 +481,18 @@ void smlDsSessionUnref(SmlDsSession *dsession)
 		if (dsession->syncCommand)
 			smlCommandUnref(dsession->syncCommand);
 		
+		if (!g_mutex_trylock(dsession->pendingMapsLock)) {
+			smlTrace(TRACE_ERROR,
+				"%s: somebody still uses this object", __func__);
+			g_mutex_lock(dsession->pendingMapsLock);
+		}
 		while (dsession->pendingMaps) {
 			SmlWriteContext *ctx = dsession->pendingMaps->data;
 			_write_context_free(ctx);
 			dsession->pendingMaps = g_list_delete_link(dsession->pendingMaps, dsession->pendingMaps);
 		}
-		
+		g_mutex_unlock(dsession->pendingMapsLock);
+		g_mutex_free(dsession->pendingMapsLock);
 		
 		while (dsession->mapItems) {
 			SmlMapItem *item = dsession->mapItems->data;
@@ -455,9 +500,12 @@ void smlDsSessionUnref(SmlDsSession *dsession)
 			dsession->mapItems = g_list_delete_link(dsession->mapItems, dsession->mapItems);
 		}
 		
-		g_mutex_free(dsession->lock);
+		if (dsession->lock)
+			g_mutex_free(dsession->lock);
+		if (dsession->write_lock)
+			g_mutex_free(dsession->write_lock);
 		
-		g_free(dsession);
+		smlSafeFree((gpointer *)&dsession);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -465,11 +513,22 @@ void smlDsSessionUnref(SmlDsSession *dsession)
 
 void smlDsSessionDispatch(SmlDsSession *dsession)
 {
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, dsession);
 	SmlError *error = NULL;
 	SmlStatus *reply = NULL;
-	
-	g_mutex_lock(dsession->lock);
-	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, dsession);
+
+	/* A dispatch function should never block on a
+	 * locked resource. This is like a busy wait
+	 * because the thread is not available for this
+	 * time which can block the whole system if two
+	 * dispatchers use the same thread.
+	 */
+	if (!g_mutex_trylock(dsession->lock))
+	{
+		smlTrace(TRACE_EXIT, "%s - DsSession is already locked", __func__);
+		return;
+	}
+	smlTrace(TRACE_INTERNAL, "%s - locked DsSession successfully", __func__);
 
 	if (dsession->alertCommand && dsession->recvAlertCallback) {
 		smlTrace(TRACE_INTERNAL, "%s: Dispatching alert", __func__);
@@ -479,7 +538,7 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 			type = SML_ERROR_REQUIRE_REFRESH;
 		dsession->recvAlertCallback = NULL;
 
-		SmlStatus *reply = smlCommandNewReply(dsession->alertCommand, type, &error);
+		reply = smlCommandNewReply(dsession->alertCommand, type, &error);
 		if (!reply)
 			goto error;
 		
@@ -492,6 +551,17 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 		smlCommandUnref(dsession->alertCommand);
 		dsession->alertCommand = NULL;
 
+		/* unlock the final event at the manager
+		 * 
+		 * If this is called from a test case then the datastore
+		 * is sometimes not registered at a SmlManager.
+		 */
+		if (dsession->server->manager && dsession->finalLock)
+		{
+			dsession->finalLock = FALSE;
+			smlManagerSessionFinalLockUnref(dsession->server->manager, dsession->session);
+		}
+
 	} else if (dsession->recvSync && dsession->recvSyncCallback) {
 		smlTrace(TRACE_INTERNAL, "%s: Dispatching sync", __func__);
 		
@@ -501,8 +571,8 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 		while (dsession->recvSync) {
 			SmlCommand *cmd = dsession->recvSync->data;
 			
-			smlTrace(TRACE_INTERNAL, "answering sync command with cmdRef %i and msgRef %i", cmd->cmdID, cmd->msgID);
-			SmlStatus *reply = smlCommandNewReply(cmd, SML_NO_ERROR, &error);
+			smlTrace(TRACE_INTERNAL, "%s: answering sync command with cmdRef %i and msgRef %i", __func__, cmd->cmdID, cmd->msgID);
+			reply = smlCommandNewReply(cmd, SML_NO_ERROR, &error);
 			if (!reply)
 				goto error;
 			
@@ -517,13 +587,26 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 		
 		dsession->syncReply = SML_NO_ERROR;
 		
-		// Now there are no more changes... do some fancy callback. Some applications will love it ;)
-		if (!dsession->recvChanges && dsession->recvEventCallback) {
-			dsession->recvEventCallback(dsession, SML_DS_EVENT_GOTCHANGES, dsession->recvEventCallbackUserdata);
-			smlTrace(TRACE_INTERNAL, "recvEventCallback no changes in recvSync callback");
-		}
+		if (!dsession->recvChanges) {
+			/* There are no changes. */
 
-		smlTrace(TRACE_EXIT, "%s", __func__);
+			/* ... do some fancy callback. Some applications will love it ;) */
+			if (dsession->recvEventCallback) {
+				dsession->recvEventCallback(dsession, SML_DS_EVENT_GOTCHANGES, dsession->recvEventCallbackUserdata);
+				smlTrace(TRACE_INTERNAL, "%s: recvEventCallback no changes in recvSync callback", __func__);
+			}
+			/* unlock the final event at the manager
+			 * 
+			 * If this is called from a test case then the datastore
+			 * is sometimes not registered at a SmlManager.
+			 */
+			if (dsession->server->manager && dsession->finalLock)
+			{
+				dsession->finalLock = FALSE;
+				smlManagerSessionFinalLockUnref(dsession->server->manager, dsession->session);
+			}
+			smlTrace(TRACE_INTERNAL, "%s: final handling done", __func__);
+		}
 	} else if (dsession->recvChanges && dsession->changesCallback) {
 		smlTrace(TRACE_INTERNAL, "%s: Dispatching changes", __func__);
 		
@@ -532,26 +615,49 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 			SmlCommand *cmd = dsession->recvChanges->data;
 			
 			if (cmd) {
-				SmlItem *item = cmd->private.change.item;
-				if (!item || (!item->source && !item->target)) {
-					smlErrorSet(&error, SML_ERROR_GENERIC, "No item");
+				if (!cmd->private.change.items ||
+				    !g_list_length(cmd->private.change.items))
+				{
+					smlErrorSet(&error, SML_ERROR_GENERIC, "No items found in command.");
 					goto error;
 				}
-				
-				/* We'd rather use the target of the change since it already the mapped
-				 * uid (if we are a client for example). If it is not given we use the
-				 * source uri. This has then to be translated by the sync engine of course */
 
-				char *data = NULL;
-				unsigned int size = 0;
-				if (!smlItemStealData(item, &data, &size, &error))
-					goto error;
-				 
-				if (!dsession->changesCallback(dsession, cmd->private.change.type, item->target ? item->target->locURI : item->source->locURI, data, size, item->contenttype, dsession->changesCallbackUserdata, &error))
-					goto error;
-						
+				/* All items handled at once because libsyncml
+				 * only supports one status code for all items.
+				 * So one failed item means that the complete
+				 * command is aborted.
+				 */
+
+				guint i;
+				for (i=0; i < g_list_length(cmd->private.change.items); i++)
+				{
+					SmlItem *item = g_list_nth_data(cmd->private.change.items, i);
+				        if (!item)
+					{
+						smlErrorSet(&error, SML_ERROR_GENERIC,
+							"Item %i of the command's item list is NULL.", i);
+						goto error;
+					}
+					/* We'd rather use the target of the change since it already the mapped
+					 * uid (if we are a client for example). If it is not given we use the
+					 * source uri. This has then to be translated by the sync engine of course */
+					if ((!item->source && !item->target)) {
+						smlErrorSet(&error, SML_ERROR_GENERIC,
+							"Cannot determine UID because source an target are missing on item %i.", i);
+						goto error;
+					}
+
+					char *data = NULL;
+					unsigned int size = 0;
+					if (!smlItemStealData(item, &data, &size, &error))
+						goto error;
+
+					if (!dsession->changesCallback(dsession, cmd->private.change.type, item->target ? item->target->locURI : item->source->locURI, data, size, item->contenttype, dsession->changesCallbackUserdata, &error))
+						goto error;
+				}
+
 				if (cmd->private.change.type == SML_CHANGE_ADD)
-					reply = smlCommandNewReply(cmd, SML_ALERT_SLOW_SYNC, &error);
+					reply = smlCommandNewReply(cmd, SML_ITEM_ADDED, &error);
 				else
 					reply = smlCommandNewReply(cmd, SML_NO_ERROR, &error);
 				
@@ -565,19 +671,30 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 			
 				smlCommandUnref(cmd);
 			}
-			
-			
+
 			dsession->recvChanges = g_list_delete_link(dsession->recvChanges, dsession->recvChanges);
 		}
 
 		// Now there are no more changes... do some fancy callback. Some applications will love it ;)
 		if (dsession->recvEventCallback) {
 			dsession->recvEventCallback(dsession, SML_DS_EVENT_GOTCHANGES, dsession->recvEventCallbackUserdata);
-			smlTrace(TRACE_INTERNAL, "recvEventCallback all changes sent in recvChanges callback");
+			smlTrace(TRACE_INTERNAL, "%s: recvEventCallback all changes sent in recvChanges callback", __func__);
 		}
+		/* unlock the final event at the manager
+		 * 
+		 * If this is called from a test case then the datastore
+		 * is sometimes not registered at a SmlManager.
+		 */
+		if (dsession->server->manager)
+		{
+			dsession->finalLock = FALSE;
+			smlManagerSessionFinalLockUnref(dsession->server->manager, dsession->session);
+		}
+		else
+			smlTrace(TRACE_INTERNAL, "%s - no manager so should be a test", __func__);
 	} else {
 		
-		smlTrace(TRACE_INTERNAL, "recvChanges: %p changesCallback: %p", dsession->recvChanges, dsession->changesCallback); 
+		smlTrace(TRACE_INTERNAL, "%s: recvChanges: %p changesCallback: %p", __func__, dsession->recvChanges, dsession->changesCallback); 
 	}
 
 	smlTrace(TRACE_EXIT, "%s()", __func__);
@@ -651,6 +768,19 @@ void smlDsSessionRecvAlert(SmlSession *session, SmlCommand *cmd, void *userdata)
 	}
 	
 	dsession->alertCommand = cmd;
+
+	/* Lock the final event at the manager.
+	 * 
+	 * If this is called from a test case then the datastore
+	 * is sometimes not registered at a SmlManager.
+	 */
+	if (dsession->server->manager)
+	{
+		smlManagerSessionFinalLockRef(dsession->server->manager, session);
+		dsession->finalLock = TRUE;
+	}
+	else
+		smlTrace(TRACE_INTERNAL, "%s - no manager so should be a test", __func__);
 	
 	g_mutex_unlock(dsession->lock);
 	
@@ -669,14 +799,49 @@ void smlDsSessionRecvSync(SmlSession *session, SmlCommand *cmd, void *userdata)
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, session, cmd, userdata);
 	SmlDsSession *dsession = userdata;
 	SmlError *error = NULL;
-	
-	g_mutex_lock(dsession->lock);
+
+	/* Create a write lock on the complete DsSession object to avoid race
+	 * conditions. If the write lock is already present then this is the
+	 * end of the sync block and the write lock must only be removed.
+	 */
+	if (g_mutex_trylock(dsession->write_lock))
+	{
+		/* write lock succeeded so lock the complete DsSession */
+		g_mutex_lock(dsession->lock);
+		smlTrace(TRACE_INTERNAL, "%s - write lock enabled", __func__);
+		/* Lock the final event at the manager.
+		 * 
+		 * If this is called from a test case then the datastore
+		 * is sometimes not registered at a SmlManager.
+		 */
+		if (dsession->server->manager)
+		{
+			smlManagerSessionFinalLockRef(dsession->server->manager, session);
+			dsession->finalLock = TRUE;
+			dsession->emptySync = TRUE;
+		}
+		else
+			smlTrace(TRACE_INTERNAL, "%s - no manager so should be a test", __func__);
+	} else {
+		/* detected an old write lock so this is the closing sync call */
+		if (dsession->finalLock && dsession->emptySync)
+		{
+			smlTrace(TRACE_INTERNAL, "%s - empty sync of moreData ?", __func__);
+			dsession->finalLock = FALSE;
+			smlManagerSessionFinalLockUnref(dsession->server->manager, session);
+		}
+		g_mutex_unlock(dsession->lock);
+		g_mutex_unlock(dsession->write_lock);
+		smlTrace(TRACE_EXIT, "%s - removed write lock", __func__);
+		return;
+	}
+
 	if (dsession->syncReply == SML_ERROR_UNKNOWN) {
-		smlTrace(TRACE_INTERNAL, "Storing sync command with cmdRef %i and msgRef %i", cmd->cmdID, cmd->msgID);
+		smlTrace(TRACE_INTERNAL, "%s: Storing sync command with cmdRef %i and msgRef %i", __func__, cmd->cmdID, cmd->msgID);
 		smlCommandRef(cmd);
 		dsession->recvSync = g_list_append(dsession->recvSync, cmd);
 	} else {
-		smlTrace(TRACE_INTERNAL, "Using stored sync reply on cmd with cmdRef %i and msgRef %i", cmd->cmdID, cmd->msgID);
+		smlTrace(TRACE_INTERNAL, "%s: Using stored sync reply on cmd with cmdRef %i and msgRef %i", __func__, cmd->cmdID, cmd->msgID);
 		SmlStatus *reply = smlCommandNewReply(cmd, dsession->syncReply, &error);
 		if (!reply)
 			goto error;
@@ -686,13 +851,13 @@ void smlDsSessionRecvSync(SmlSession *session, SmlCommand *cmd, void *userdata)
 		
 		smlStatusUnref(reply);
 	}
-	g_mutex_unlock(dsession->lock);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return;
 	
 error:
 	g_mutex_unlock(dsession->lock);
+	g_mutex_unlock(dsession->write_lock);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(&error));
 	smlErrorDeref(&error);
 }
@@ -701,12 +866,16 @@ void smlDsSessionRecvChange(SmlSession *session, SmlCommand *cmd, void *userdata
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, session, cmd, userdata);
 	SmlDsSession *dsession = userdata;
-	
-	g_mutex_lock(dsession->lock);
+
+	/* It is not necessary to lock the DsSesion object
+	 * because RecvSync already enabled a global write_lock.
+	 */
 	dsession->recvChanges = g_list_append(dsession->recvChanges, cmd);
 	smlCommandRef(cmd);
-	g_mutex_unlock(dsession->lock);
-	
+
+	/* This is important for fast and consistent final signalling. */
+	dsession->emptySync = FALSE;
+
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -730,10 +899,24 @@ void smlDsSessionRecvMap(SmlSession *session, SmlCommand *cmd, void *userdata)
 	GList *m = NULL;
 	for (m = cmd->private.map.items; m; m = m->next) {
 		SmlMapItem *item = m->data;
-		SmlWriteContext *ctx = _write_context_find(dsession, item->target->locURI, SML_CHANGE_ADD);
-		if (ctx) {
-			ctx->newuid = g_strdup(item->source->locURI);
-			_write_context_dispatch(dsession, ctx);
+
+		if (dsession->recvMappingCallback) {
+			dsession->recvMappingCallback(
+					dsession,
+					item->target,
+					item->source,
+					dsession->recvMappingCallbackUserdata);
+		} else {
+			SmlWriteContext *ctx = _write_context_find(dsession, item->target->locURI, SML_CHANGE_ADD);
+			if (ctx) {
+				ctx->newuid = g_strdup(item->source->locURI);
+				_write_context_dispatch(dsession, ctx);
+			} else {
+				smlTrace(TRACE_ERROR, "%s: Unknown map ... %s => %s",
+					__func__,
+					VA_STRING(item->target->locURI),
+					VA_STRING(item->source->locURI));
+			}
 		}
 	}
 	
@@ -762,7 +945,7 @@ error:
  */
 void smlDsSessionGetAlert(SmlDsSession *dsession, SmlDsSessionAlertCb callback, void *userdata)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, dsession, callback, userdata);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, dsession, callback, userdata);
 	smlAssert(dsession);
 	smlAssert(callback);
 	
@@ -788,8 +971,9 @@ void smlDsSessionGetAlert(SmlDsSession *dsession, SmlDsSessionAlertCb callback,
  */
 SmlBool smlDsSessionSendAlert(SmlDsSession *dsession, SmlAlertType type, const char *last, const char *next, SmlStatusReplyCb callback, void *userdata, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %s, %p)", __func__, dsession, type, last, next, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %s, %p)", __func__, dsession, type, VA_STRING(last), VA_STRING(next), error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	SmlCommand *alert = smlCommandNewAlert(type, dsession->target, dsession->location, next, last, NULL, error);
 	if (!alert)
@@ -811,6 +995,17 @@ error:
 	return FALSE;
 }
 
+void smlDsSessionGetMapping(
+		SmlDsSession *dsession,
+		SmlDsSessionMapCb mapCallback,
+		void *userdata)
+{
+	smlTrace(TRACE_ENTRY, "%s", __func__);
+	dsession->recvMappingCallback = mapCallback;
+	dsession->recvMappingCallbackUserdata = userdata;
+	smlTrace(TRACE_EXIT, "%s", __func__);
+}
+
 /** @brief Gets a already received sync command
  * 
  * This function will get a already received sync command or register a callback that will be
@@ -861,7 +1056,7 @@ void smlDsSessionGetEvent(SmlDsSession *dsession, SmlDsSessionEventCb eventCallb
 	dsession->recvEventCallback = eventCallback;
 	dsession->recvEventCallbackUserdata = userdata;
 	
-	//smlDsSessionDispatch(dsession);
+	smlDsSessionDispatch(dsession);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -883,6 +1078,7 @@ SmlBool smlDsSessionSendSync(SmlDsSession *dsession, unsigned int num_changes, S
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p, %p, %p)", __func__, dsession, num_changes, callback, userdata, error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	if (dsession->syncCommand) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "There already was a sync command started");
@@ -910,8 +1106,9 @@ error:
 //Send a change to the remote side
 SmlBool smlDsSessionQueueChange(SmlDsSession *dsession, SmlChangeType type, const char *uid, const char *data, unsigned int size, const char *contenttype, SmlDsSessionWriteCb callback, void *userdata, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %p, %i, %s, %p, %p, %p)", __func__, dsession, type, uid, data, size, contenttype, callback, userdata, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %p, %i, %s, %p, %p, %p)", __func__, dsession, type, VA_STRING(uid), data, size, VA_STRING(contenttype), callback, userdata, error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	if (!dsession->syncCommand) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "You have to start a sync command first");
@@ -932,7 +1129,9 @@ SmlBool smlDsSessionQueueChange(SmlDsSession *dsession, SmlChangeType type, cons
 	ctx->type = type;
 	ctx->session = dsession;
 	
+	g_mutex_lock(dsession->pendingMapsLock);
 	dsession->pendingMaps = g_list_append(dsession->pendingMaps, ctx);
+	g_mutex_unlock(dsession->pendingMapsLock);
 	
 	if (!smlSessionSendCommand(dsession->session, cmd, dsession->syncCommand, _change_reply, ctx, error))
 		goto error_free_ctx;
@@ -943,8 +1142,8 @@ SmlBool smlDsSessionQueueChange(SmlDsSession *dsession, SmlChangeType type, cons
 	return TRUE;
 
 error_free_ctx:
-	g_free(ctx->uid);
-	g_free(ctx);
+	smlSafeCFree(&(ctx->uid));
+	smlSafeFree((gpointer *)&ctx);
 error_free_cmd:
 	smlCommandUnref(cmd);
 error:
@@ -962,6 +1161,7 @@ SmlBool smlDsSessionCloseSync(SmlDsSession *dsession, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, dsession, error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	if (!dsession->syncCommand) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "There already was a sync command started");
@@ -984,8 +1184,9 @@ error:
 
 SmlBool smlDsSessionQueueMap(SmlDsSession *dsession, const char *uid, const char *newuid, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %s, %s, %p)", __func__, dsession, uid, newuid, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %s, %s, %p)", __func__, dsession, VA_STRING(uid), VA_STRING(newuid), error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	SmlMapItem *item = smlMapItemNew(uid, newuid, error);
 	if (!item)
@@ -1014,6 +1215,7 @@ SmlBool smlDsSessionCloseMap(SmlDsSession *dsession, SmlStatusReplyCb callback,
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, dsession, callback, userdata, error);
 	smlAssert(dsession);
+	CHECK_ERROR_REF
 	
 	if (!dsession->mapItems) {
 		smlTrace(TRACE_EXIT, "%s: No mapitems", __func__);
@@ -1086,7 +1288,7 @@ static void _recv_manager_alert(SmlSession *session, SmlCommand *cmd, void *user
 	return;
 
 error_free_dsession:
-	g_free(dsession);
+	smlSafeFree((gpointer *)&dsession);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(&error));
 	if (error)
@@ -1098,8 +1300,32 @@ static void _recv_manager_san(SmlSession *session, SmlCommand *cmd, void *userda
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, session, cmd, userdata);
 	SmlDsServer *server = userdata;
 	SmlError *error = NULL;
-	
-	if (server->sanCallback) {
+
+	/* ensure that the session is in client mode */
+	session->sessionType = SML_SESSION_TYPE_CLIENT;
+
+	/* fix target */
+	smlLocationUnref(server->target);
+	server->target = cmd->source;
+	smlLocationRef(server->target);
+
+	/* handle the SAN */
+	if (server->sanSessionCallback) {
+		SmlErrorType type = server->sanSessionCallback(
+						server,
+						session,
+						cmd->private.alert.type,
+						server->sanSessionCallbackUserdata);
+		
+		SmlStatus *reply = smlCommandNewReply(cmd, type, &error);
+		if (!reply)
+			goto error;
+		
+		if (!smlSessionSendReply(session, reply, &error))
+			goto error;
+			
+		smlStatusUnref(reply);
+	} else if (server->sanCallback) {
 		SmlErrorType type = server->sanCallback(server, cmd->private.alert.type, server->sanCallbackUserdata);
 		
 		SmlStatus *reply = smlCommandNewReply(cmd, type, &error);
@@ -1111,9 +1337,9 @@ static void _recv_manager_san(SmlSession *session, SmlCommand *cmd, void *userda
 			
 		smlStatusUnref(reply);
 	} else {
-		smlTrace(TRACE_INTERNAL, "SAN ignored");
+		smlTrace(TRACE_INTERNAL, "%s: SAN ignored", __func__);
 		
-		SmlStatus *reply = smlCommandNewReply(cmd, SML_ERROR_NOT_SUPPORTED, &error);
+		SmlStatus *reply = smlCommandNewReply(cmd, SML_ERROR_NOT_IMPLEMENTED, &error);
 		if (!reply)
 			goto error;
 		
@@ -1137,6 +1363,7 @@ SmlBool smlDsServerRegister(SmlDsServer *server, SmlManager *manager, SmlError *
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, server, manager, error);
 	smlAssert(server);
 	smlAssert(manager);
+	CHECK_ERROR_REF
 	
 	if (!smlManagerObjectRegister(manager, SML_COMMAND_TYPE_ALERT, NULL, server->location, NULL, NULL, _recv_manager_alert, NULL, server, error))
 		goto error;
@@ -1160,4 +1387,3 @@ SmlLocation *smlDsSessionGetTarget(SmlDsSession *dsession)
 	return dsession->target;
 }
 
-/*@}*/
