@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
 
+/**
+ * @defgroup SmlObjectDatastoreServer SyncML Datastore Server Object
+ * @ingroup PublicObjectAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_DS_SERVER_H_
 #define _SML_DS_SERVER_H_
 
@@ -29,13 +38,14 @@ typedef enum {
 	SML_DS_EVENT_COMMITEDCHANGES = 1
 } SmlDsEvent;
 
-typedef SmlErrorType (* SmlDsServerSanCb) (SmlDsServer *dsserver, SmlAlertType type, void *userdata);
-
 typedef void (* SmlDsSessionConnectCb) (SmlDsSession *dsession, void *userdata);
 typedef SmlBool (* SmlDsSessionAlertCb) (SmlDsSession *dsession, SmlAlertType type, const char *last, const char *next, void *userdata);
 typedef void (* SmlDsSessionSyncCb) (SmlDsSession *dsession, unsigned int numchanges, void *userdata);
 typedef void (* SmlDsSessionEventCb) (SmlDsSession *dsession, SmlDsEvent event, void *userdata);
 typedef SmlBool (* SmlDsSessionChangesCb) (SmlDsSession *dsession, SmlChangeType type, const char *uid, char *data, unsigned int size, const char *contenttype, void *userdata, SmlError **error);
+/* FIXME: newuid must be removed before next API break */
+/* FIXME: an internal mapping callback already exists */
+/* FIXME: DEPRECATED */
 typedef void (* SmlDsSessionWriteCb) (SmlDsSession *dsession, SmlStatus *status, const char *newuid, void *userdata);
 
 SmlDsServer *smlDsServerNew(const char *type, SmlLocation *location, SmlError **error);
@@ -46,7 +56,6 @@ SmlBool smlDsServerRegister(SmlDsServer *server, SmlManager *manager, SmlError *
 SmlBool smlDsServerAddSan(SmlDsServer *server, SmlNotification *san, SmlError **error);
 void smlDsServerReset(SmlDsServer *server);
 void smlDsServerSetConnectCallback(SmlDsServer *server, SmlDsSessionConnectCb callback, void *userdata);
-void smlDsServerSetSanCallback(SmlDsServer *server, SmlDsServerSanCb callback, void *userdata);
 const char *smlDsServerGetLocation(SmlDsServer *server);
 const char *smlDsServerGetContentType(SmlDsServer *server);
 SmlDsSession *smlDsServerSendAlert(SmlDsServer *server, SmlSession *session, SmlAlertType type, const char *last, const char *next, SmlStatusReplyCb callback, void *userdata, SmlError **error);
@@ -71,4 +80,15 @@ const char *smlDsSessionGetContentType(SmlDsSession *dsession);
 SmlDsServer *smlDsSessionGetServer(SmlDsSession *dsession);
 SmlLocation *smlDsSessionGetTarget(SmlDsSession *dsession);
 
+/* multi session safe SAN handling */
+
+typedef SmlErrorType (* SmlDsServerSanSessionCb) (SmlDsServer *dsserver, SmlSession *session, SmlAlertType type, void *userdata);
+void smlDsServerSetSanSessionCallback(SmlDsServer *server, SmlDsServerSanSessionCb callback, void *userdata);
+
+/* deprecated SAN handling */
+
+typedef SmlErrorType (* SmlDsServerSanCb) (SmlDsServer *dsserver, SmlAlertType type, void *userdata);
+void smlDsServerSetSanCallback(SmlDsServer *server, SmlDsServerSanCb callback, void *userdata) LIBSYNCML_DEPRECATED;
+
 #endif //_SML_DS_SERVER_H_
+/*@}*/
