@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,9 +19,19 @@
  *
  */
 
+/**
+ * @defgroup SmlObjectDatastoreServerPrivate SyncML Datastore Server Object Internals
+ * @ingroup PrivateObjectAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_DS_SERVER_INTERNALS_H_
 #define _SML_DS_SERVER_INTERNALS_H_
 
+typedef void (* SmlDsSessionMapCb) (SmlDsSession *dsession, SmlLocation *orig, SmlLocation *mapped, void *userdata);
+
 typedef struct SmlWriteContext {
 	SmlDsSessionWriteCb callback;
 	char *uid;
@@ -61,12 +72,14 @@ struct SmlDsSession {
 	GList *recvSync;
 	GList *recvChanges;
 	SmlErrorType syncReply;
+	SmlBool finalLock;
+	SmlBool emptySync;
 	
 	/** Callback that will receive the reply to the sync command */
 	SmlStatusReplyCb sentSyncCallback;
 	void *sentSyncCallbackUserdata;
 	
-	/** Callback that will infomration about the Ds Session events */
+	/** Callback that will information about the Ds Session events */
 	SmlDsSessionEventCb recvEventCallback;
 	void *recvEventCallbackUserdata;
 
@@ -74,9 +87,14 @@ struct SmlDsSession {
 	SmlCommand *syncCommand;
 	
 	GList *pendingMaps;
+	GMutex *pendingMapsLock;
 	GList *mapItems;
-	//SmlCommand *mapCommand;
-	
+
+	/** Callback that signals mappings */
+	SmlDsSessionMapCb recvMappingCallback;
+	void *recvMappingCallbackUserdata;
+
+	GMutex *write_lock;
 	GMutex *lock;
 };
 
@@ -92,6 +110,8 @@ struct SmlDsServer {
 	SmlDsSessionConnectCb connectCallback;
 	void *connectCallbackUserdata;
 	
+	SmlDsServerSanSessionCb sanSessionCallback;
+	void *sanSessionCallbackUserdata;
 	SmlDsServerSanCb sanCallback;
 	void *sanCallbackUserdata;
 };
@@ -105,4 +125,7 @@ SmlDsSession *smlDsServerRecvAlert(SmlDsServer *server, SmlSession *session, Sml
 SmlDsSession *smlDsSessionNew(SmlDsServer *server, SmlSession *session, SmlError **error);
 void smlDsSessionFree(SmlDsSession *dsession);
 
+void smlDsSessionGetMapping(SmlDsSession *dsession, SmlDsSessionMapCb mapCallback, void *userdata);
+
 #endif //_SML_DS_SERVER_INTERNALS_H_
+/*@}*/
