@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +25,7 @@
 #include <libsyncml/sml_elements_internals.h>
 #include <libsyncml/sml_command_internals.h>
 #include <libsyncml/sml_session_internals.h>
+#include "libsyncml/sml_error_internals.h"
 
 #ifdef ENABLE_WBXML
 
@@ -34,7 +36,7 @@
 #include "sml_wbxml.h"
 #include "sml_wbxml_internals.h"
 
-SmlBool smlWbxmlConvertTo(WBXMLConvXML2WBXMLParams *params, const char *input, char **output, unsigned int *outputLen, SmlError** error)
+SmlBool smlWbxmlConvertTo(WBXMLConvXML2WBXMLParams *params, const char *input, char **output, size_t *outputLen, SmlError** error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %p)", __func__, params, input, output, outputLen, error);
 	smlAssert(input);
@@ -56,13 +58,13 @@ error:
 	return FALSE;
 }
 
-SmlBool smlWbxmlConvertFrom(WBXMLConvWBXML2XMLParams *params, const char *input, unsigned int inputLen, char **output, SmlError** error)
+SmlBool smlWbxmlConvertFrom(WBXMLConvWBXML2XMLParams *params, const char *input, size_t inputLen, char **output, size_t *outputLen, SmlError** error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p)", __func__, params, input, inputLen, output, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p, %p)", __func__, params, input, inputLen, output, outputLen, error);
 	WBXMLError wberror;
 
 	smlTrace(TRACE_INTERNAL, "WBXML2 VERSION: %s", WBXML_LIB_VERSION);
-	wberror = wbxml_conv_wbxml2xml((WB_UTINY*)input, inputLen, (WB_UTINY**)output, params);
+	wberror = wbxml_conv_wbxml2xml_withlen((WB_UTINY*)input, inputLen, (WB_UTINY**)output, (WB_ULONG*)outputLen,  params);
 	if (wberror != WBXML_OK)
 		goto error;
 	
@@ -78,29 +80,37 @@ error:
 static SmlBool smlWbxmlParserStart(SmlXmlParser *parser, const char *data, unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, parser, data, size, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(data);
 	smlAssert(size);
 	
 	char *bin = smlPrintBinary(data, size);
 	smlTrace(TRACE_INTERNAL, "Wbxml input: %s", bin);
-	g_free(bin);
+	smlSafeCFree(&bin);
 	smlLog("received-%i.wbxml", data, size);
-	
+
+	/* Ignorable whitespaces must be preserved because sometimes
+	 * a buggy mobile does not the its data objects correctly
+	 * encapsulated into CDATA.
+	 */
 	char *buffer = NULL;
-	WBXMLConvWBXML2XMLParams params = {WBXML_ENCODER_XML_GEN_COMPACT, WBXML_LANG_UNKNOWN, 0, FALSE};
-	if (!smlWbxmlConvertFrom(&params, data, size, &buffer, error))
+	size_t buffer_size = 0;
+	WBXMLConvWBXML2XMLParams params = {WBXML_ENCODER_XML_GEN_COMPACT, WBXML_LANG_UNKNOWN, 0, TRUE};
+	if (!smlWbxmlConvertFrom(&params, data, size, &buffer, &buffer_size, error))
 		goto error;
-	unsigned int buffer_size = strlen(buffer);
 	smlTrace(TRACE_INTERNAL, "converted XML message: %s", buffer);
 	
 	if (!smlXmlParserStart(parser, buffer, buffer_size, error))
 		goto error;
+	smlSafeCFree(&buffer);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
 error:
+	if (buffer)
+		smlSafeCFree(&buffer);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
@@ -108,6 +118,7 @@ error:
 SmlXmlParser *smlWbxmlParserNew(SmlParserFunctions *functions, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, functions, error);
+	CHECK_ERROR_REF
 	smlAssert(functions);
 	
 	SmlXmlParser *parser = smlXmlParserNew(functions, error);
@@ -127,6 +138,7 @@ error:
 SmlBool smlWbxmlAssemblerRun(SmlXmlAssembler *assm, char **data, unsigned int *size, SmlBool *end, SmlBool final, unsigned int maxsize, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %i, %i, %p)", __func__, assm, data, size, end, final, maxsize, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(data);
 	smlAssert(size);
@@ -136,16 +148,17 @@ SmlBool smlWbxmlAssemblerRun(SmlXmlAssembler *assm, char **data, unsigned int *s
 	if (!smlXmlAssemblerRun(assm, &buffer, &buffer_size, end, final, 0, error))
 		goto error;
 	
-	WBXMLConvXML2WBXMLParams params = {WBXML_VERSION_12, FALSE, FALSE};
+	WBXMLConvXML2WBXMLParams params = {WBXML_VERSION_12, FALSE, FALSE, FALSE};
 	const char *opt = smlAssemblerGetOption(assm->assembler, "USE_STRTABLE");
 	if (opt && atoi(opt))
 		params.use_strtbl = TRUE;
 	if (!smlWbxmlConvertTo(&params, buffer, data, size, error))
 		goto error;
+	smlSafeCFree(&buffer);
 	
 	char *hex = smlPrintHex(*data, *size);
 	smlTrace(TRACE_INTERNAL, "Wbxml assembled: %s", hex);
-	g_free(hex);
+	smlSafeCFree(&hex);
 
 	smlLog("sent-%i.wbxml", *data, *size);
 	
@@ -153,6 +166,8 @@ SmlBool smlWbxmlAssemblerRun(SmlXmlAssembler *assm, char **data, unsigned int *s
 	return TRUE;
 
 error:
+	if (buffer)
+		smlSafeCFree(&buffer);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
@@ -160,6 +175,7 @@ error:
 unsigned int smlWbxmlAssemblerCheckSize(SmlXmlAssembler *assm, SmlBool headeronly, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, assm, headeronly, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	
 	unsigned int size = 0;
@@ -169,21 +185,21 @@ unsigned int smlWbxmlAssemblerCheckSize(SmlXmlAssembler *assm, SmlBool headeronl
 	if (!smlXmlAssemblerRunFull(assm, &buffer, &buffer_size, NULL, TRUE, FALSE, 0, error))
 		goto error;
 	
-	WBXMLConvXML2WBXMLParams params = {WBXML_VERSION_12, FALSE, FALSE};
+	WBXMLConvXML2WBXMLParams params = {WBXML_VERSION_12, FALSE, FALSE, FALSE};
 	const char *opt = smlAssemblerGetOption(assm->assembler, "USE_STRTABLE");
 	if (opt && atoi(opt))
 		params.use_strtbl = TRUE;
 	if (!smlWbxmlConvertTo(&params, buffer, &data, &size, error))
 		goto error_free_buffer;
 	
-	g_free(data);
-	g_free(buffer);
+	smlSafeCFree(&data);
+	smlSafeCFree(&buffer);
 	
 	smlTrace(TRACE_EXIT, "%s: %i", __func__, size);
 	return size;
 
 error_free_buffer:
-	g_free(buffer);
+	smlSafeCFree(&buffer);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return 0;
@@ -199,6 +215,7 @@ error:
 SmlXmlAssembler *smlWbxmlAssemblerNew(SmlAssembler *assembler, SmlAssemblerFunctions *functions, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assembler, functions, error);
+	CHECK_ERROR_REF
 	
 	SmlXmlAssembler *assm = smlXmlAssemblerNew(assembler, functions, error);
 	if (!assm)
