@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
  
+/**
+ * @defgroup XmlAssembler XML Assembler
+ * @ingroup XmlPublic
+ * @brief Interfaces to assemble SyncML XML messages
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_XML_ASSM_H_
 #define _SML_XML_ASSM_H_
 
@@ -39,4 +48,11 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 
 SmlBool smlXmlDevInfAssemble(SmlDevInf *devinf, SmlDevInfVersion version, char **data, unsigned int *size, SmlError **error);
 
+/* This is necessary if the first message results in an error 407.
+ * This error means authentication required.
+ * After such an error the previous commands must be sent again.
+ */
+void smlXmlAssemblerRestoreCommands(SmlXmlAssembler *assm);
+
 #endif //_SML_XML_ASSM_H_
+/*@}*/
