@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
  
+/**
+ * @defgroup XmlAssemblerPrivate XML Assembler Internals
+ * @ingroup XmlPrivate
+ * @brief XML assembler details only known inside the internal XML/WBXML API.
+ * 
+ */
+/*@{*/
+ 
 #ifndef _SML_XML_ASSM_INTERNALS_H_
 #define _SML_XML_ASSM_INTERNALS_H_
 
@@ -61,6 +70,7 @@ struct SmlXmlAssembler {
 	xmlBuffer *header_buffer;
 	GList *statuses;
 	GList *commands;
+	GList *last_commands;
 	GList *parent_commands;
 	GList *parent_buffers;
 	SmlSession *session;
@@ -82,3 +92,4 @@ SmlBool smlAssemblerAddID(SmlAssembler *assm, const char *name, unsigned int id,
 SmlBool smlAssemblerAddData(SmlAssembler *assm, const char *name, const char *value, SmlError **error);
 
 #endif //_SML_XML_ASSM_INTERNALS_H_
+/*@}*/
