@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
 
+/**
+ * @defgroup SmlDefines SyncML Defines
+ * @ingroup PublicLowLevelAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_DEFINES_H
 #define _SML_DEFINES_H
 
@@ -30,8 +39,12 @@
 #define SML_ELEMENT_SESSIONID "SessionID"
 #define SML_ELEMENT_TARGET "Target"
 #define SML_ELEMENT_SOURCE "Source"
+#define SML_ELEMENT_SOURCE_PARENT "SourceParent"
+#define SML_ELEMENT_TARGET_PARENT "TargetParent"
 #define SML_ELEMENT_LOCURI "LocURI"
 #define SML_ELEMENT_LOCNAME "LocName"
+#define SML_ELEMENT_RESPURI "RespURI"
+#define SML_ELEMENT_NORESP "NoResp"
 #define SML_ELEMENT_SYNC "Sync"
 #define SML_ELEMENT_STATUS "Status"
 #define SML_ELEMENT_CMDID "CmdID"
@@ -104,6 +117,7 @@
 #define SML_ELEMENT_SIZE "Size"
 #define SML_ELEMENT_SUPPORTLARGEOBJS "SupportLargeObjs"
 #define SML_ELEMENT_SUPPORTNUMBEROFCHANGES "SupportNumberOfChanges"
+#define SML_ELEMENT_SUPPORT_HIERARCHICAL_SYNC "SupportHierarchicalSync"
 #define SML_ELEMENT_SWV "SwV"
 #define SML_ELEMENT_SYNCCAP "SyncCap"
 #define SML_ELEMENT_SYNCTYPE "SyncType"
@@ -193,6 +207,9 @@ typedef enum {
 #define SML_ELEMENT_WBXML "application/vnd.syncml+wbxml"
 #define SML_ELEMENT_XML "application/vnd.syncml+xml"
 #define SML_ELEMENT_SAN "application/vnd.syncml.ds.notification"
+#define SML_ELEMENT_APPLICATION_OMA_DS_EMAIL "application/vnd.omads-email+xml"
+#define SML_ELEMENT_APPLICATION_OMA_DS_FILE "application/vnd.omads-file+xml"
+#define SML_ELEMENT_APPLICATION_OMA_DS_FOLDER "application/vnd.omads-folder+xml"
 
 #define SML_NAMESPACE_METINF "syncml:metinf"
 #define SML_NAMESPACE_DEVINF "syncml:devinf"
@@ -208,6 +225,32 @@ typedef enum {
 #define SML_AUTH_MD5 "syncml:auth-md5"
 #define SML_BASE64 "b64"
 
+/* Transport configuration defines */
+
+#define SML_TRANSPORT_CONFIG_PROXY             "PROXY"
+#define SML_TRANSPORT_CONFIG_USERNAME          "USERNAME"
+#define SML_TRANSPORT_CONFIG_PASSWORD          "PASSWORD"
+#define SML_TRANSPORT_CONFIG_SSL_CA_FILE       "SSL_CA_FILE"
+#define SML_TRANSPORT_CONFIG_PORT              "PORT"
+#define SML_TRANSPORT_CONFIG_URL               "URL"
+#define SML_TRANSPORT_CONFIG_SSL_KEY           "SSL_KEY"
+#define SML_TRANSPORT_CONFIG_SSL_SERVER_CERT   "SSL_SERVER_CERT"
+#define SML_TRANSPORT_CONFIG_BLUETOOTH_ADDRESS "BLUETOOTH_ADDRESS"
+#define SML_TRANSPORT_CONFIG_BLUETOOTH_CHANNEL "BLUETOOTH_CHANNEL"
+#define SML_TRANSPORT_CONFIG_IRDA_SERVICE      "IRDA_SERVICE"
+
+#define SML_TRANSPORT_CONFIG_AT_COMMAND      "AT_COMMAND"
+#define SML_TRANSPORT_CONFIG_AT_MANUFACTURER "AT_MANUFACTURER"
+#define SML_TRANSPORT_CONFIG_AT_MODEL        "AT_MODEL"
+
+#define SML_TRANSPORT_CONFIG_DATASTORE         "DATASTORE"
+#define SML_TRANSPORT_CONFIG_DATASTORE_EVENT   "EVENT"
+#define SML_TRANSPORT_CONFIG_DATASTORE_TODO    "TODO"
+#define SML_TRANSPORT_CONFIG_DATASTORE_CONTACT "CONTACT"
+#define SML_TRANSPORT_CONFIG_DATASTORE_NOTE    "NOTE"
+
+/* Enumeration types */
+
 typedef enum  {
 	SML_DEVINF_VERSION_UNKNOWN = 0,
 	SML_DEVINF_VERSION_10 = 1,
@@ -319,3 +362,5 @@ typedef enum {
 #endif
 
 #endif //_SML_DEFINES_H
+
+/*@}*/
