@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,28 +25,12 @@
 #include "sml_parse_internals.h"
 #include "sml_elements_internals.h"
 #include "sml_command_internals.h"
-
-/**
- * @defgroup GroupIDPrivate Group Description Internals
- * @ingroup ParentGroupID
- * @brief The private part
- * 
- */
-/*@{*/
-
-/*@}*/
-
-/**
- * @defgroup GroupID Group Description
- * @ingroup ParentGroupID
- * @brief What does this group do?
- * 
- */
-/*@{*/
+#include "sml_error_internals.h"
 
 SmlLocation *smlLocationNew(const char *locURI, const char *locName, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, locURI, locName, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, VA_STRING(locURI), VA_STRING(locName), error);
+	CHECK_ERROR_REF
 	
 	if (!locURI) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "No locURI was given");
@@ -85,15 +70,15 @@ void smlLocationUnref(SmlLocation *loc)
 	smlAssert(loc);
 	
 	if (g_atomic_int_dec_and_test(&(loc->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (loc->locURI)
-			g_free(loc->locURI);
+			smlSafeCFree(&(loc->locURI));
 			
 		if (loc->locName)
-			g_free(loc->locName);
+			smlSafeCFree(&(loc->locName));
 			
-		g_free(loc);
+		smlSafeFree((gpointer *)&loc);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -105,6 +90,15 @@ const char *smlLocationGetURI(SmlLocation *loc)
 	return loc->locURI;
 }
 
+void smlLocationSetURI(SmlLocation *loc, const char *uri)
+{
+	smlAssert(loc);
+	smlAssert(uri);
+	if (loc->locURI)
+		smlSafeCFree(&(loc->locURI));
+	loc->locURI = g_strdup(uri);
+}
+
 const char *smlLocationGetName(SmlLocation *loc)
 {
 	smlAssert(loc);
@@ -116,13 +110,14 @@ void smlLocationSetName(SmlLocation *loc, const char *name)
 	smlAssert(loc);
 	smlAssert(name);
 	if (loc->locName)
-		g_free(loc->locName);
+		smlSafeCFree(&(loc->locName));
 	loc->locName = g_strdup(name);
 }
 
 SmlLocation *smlLocationClone(SmlLocation *source, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, source, error);
+	CHECK_ERROR_REF
 	smlAssert(source);
 	
 	SmlLocation *location = smlTryMalloc0(sizeof(SmlLocation), error);
@@ -147,10 +142,10 @@ void smlLocationCopy(SmlLocation *source, SmlLocation *target)
 	smlAssert(target);
 	
 	if (target->locURI)
-		g_free(target->locURI);
+		smlSafeCFree(&(target->locURI));
 		
 	if (target->locName)
-		g_free(target->locName);
+		smlSafeCFree(&(target->locName));
 		
 	target->locURI = g_strdup(source->locURI);
 	target->locName = g_strdup(source->locName);
@@ -167,9 +162,9 @@ char *strreplace(const char *input, const char *needle, const char *replacement)
 	while (g_strrstr(output, needle)) {
 		char *buffer = g_strndup(output, g_strrstr(output, needle) - output);
 		char *buffer2 = g_strconcat(buffer, replacement ? replacement : "", g_strrstr(output, needle) + strlen(needle), NULL);
-		g_free(output);
+		smlSafeCFree(&output);
 		output = buffer2;
-		g_free(buffer);
+		smlSafeCFree(&buffer);
 	}
 	
 	return output;
@@ -177,25 +172,33 @@ char *strreplace(const char *input, const char *needle, const char *replacement)
 
 char *normalizeUrl(const char *url)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s)", __func__, url);
+	smlTrace(TRACE_ENTRY, "%s(%s)", __func__, VA_STRING(url));
 	
 	if (!url)
 		return NULL;
 	
 	char *buffer = strreplace(url, "./", "");
 	char *buffer2 = strreplace(buffer, "//", "/");
-	g_free(buffer);
+	smlSafeCFree(&buffer);
 	
-	if (buffer2[strlen(buffer2) - 1] == '/')
+	if (strlen(buffer2) > 0 && buffer2[strlen(buffer2) - 1] == '/')
 		buffer2[strlen(buffer2) - 1] = 0;
 	
-	smlTrace(TRACE_EXIT, "%s: %s", __func__, buffer2);
+	smlTrace(TRACE_EXIT, "%s: %s", __func__, VA_STRING(buffer2));
 	return buffer2;
 }
 
 SmlBool smlLocationCompare(SmlLocation *objectroot, SmlLocation *object, SmlLocation *urlroot, SmlLocation *url)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p(%s), %p(%s), %p(%s), %p(%s))", __func__, objectroot, objectroot ? objectroot->locURI : "null", object, object ? object->locURI : "null", urlroot, urlroot ? urlroot->locURI : "null", url, url ? url->locURI : "null");
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, objectroot, object, urlroot, url);
+	if (objectroot)
+		smlTrace(TRACE_INTERNAL, "%s: objectroot: %s", __func__, VA_STRING(objectroot->locURI));
+	if (object)
+		smlTrace(TRACE_INTERNAL, "%s: object: %s", __func__, VA_STRING(object->locURI));
+	if (urlroot)
+		smlTrace(TRACE_INTERNAL, "%s: urlroot: %s", __func__, VA_STRING(urlroot->locURI));
+	if (url)
+		smlTrace(TRACE_INTERNAL, "%s: url: %s", __func__, VA_STRING(url->locURI));
 	char *cmpobjurl = NULL;
 	char *cmpurl = NULL;
 	char *buffer = NULL;
@@ -218,8 +221,8 @@ SmlBool smlLocationCompare(SmlLocation *objectroot, SmlLocation *object, SmlLoca
 		buffer2 = normalizeUrl(url->locURI);
 		buffer = g_strconcat(urlroot->locURI, "/", buffer2, NULL);
 		cmpurl = normalizeUrl(buffer);
-		g_free(buffer);
-		g_free(buffer2);
+		smlSafeCFree(&buffer);
+		smlSafeCFree(&buffer2);
 	}
 	
 	if (g_path_is_absolute(object->locURI) || !objectroot)
@@ -228,20 +231,20 @@ SmlBool smlLocationCompare(SmlLocation *objectroot, SmlLocation *object, SmlLoca
 		buffer2 = normalizeUrl(object->locURI);
 		buffer = g_strconcat(objectroot->locURI, "/", buffer2, NULL);
 		cmpobjurl = normalizeUrl(buffer);
-		g_free(buffer);
-		g_free(buffer2);
+		smlSafeCFree(&buffer);
+		smlSafeCFree(&buffer2);
 	}
 	
 	
-	smlTrace(TRACE_INTERNAL, "Comparing %s and %s", cmpobjurl, cmpurl);
+	smlTrace(TRACE_INTERNAL, "%s: Comparing %s and %s", __func__, VA_STRING(cmpobjurl), VA_STRING(cmpurl));
 	
 	if (strcmp(cmpobjurl, cmpurl))
 		ret = FALSE;
 	else
 		ret = TRUE;
 	
-	g_free(cmpobjurl);
-	g_free(cmpurl);
+	smlSafeCFree(&cmpobjurl);
+	smlSafeCFree(&cmpurl);
 	
 	smlTrace(TRACE_EXIT, "%s: %i", __func__, ret);
 	return ret;
@@ -256,7 +259,8 @@ SmlBool smlLocationIsRelative(SmlLocation *location)
 
 SmlAnchor *smlAnchorNew(const char *last, const char *next, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, last, next, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, VA_STRING(last), VA_STRING(next), error);
+	CHECK_ERROR_REF
 	
 	SmlAnchor *anchor = smlTryMalloc0(sizeof(SmlAnchor), error);
 	if (!anchor)
@@ -278,12 +282,12 @@ void smlAnchorFree(SmlAnchor *anchor)
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, anchor);
 	smlAssert(anchor);
 	if (anchor->last)
-		g_free(anchor->last);
+		smlSafeCFree(&(anchor->last));
 		
 	if (anchor->next)
-		g_free(anchor->next);
+		smlSafeCFree(&(anchor->next));
 		
-	g_free(anchor);
+	smlSafeFree((gpointer *)&anchor);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -293,10 +297,10 @@ void smlHeaderFree(SmlHeader *header)
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, header);
 	
 	if (header->sessionID)
-		g_free(header->sessionID);
+		smlSafeCFree(&(header->sessionID));
 
 	if (header->emi)
-		g_free(header->emi);
+		smlSafeCFree(&(header->emi));
 
 	if (header->source)
 		smlLocationUnref(header->source);
@@ -304,14 +308,18 @@ void smlHeaderFree(SmlHeader *header)
 	if (header->target)
 		smlLocationUnref(header->target);
 	
-	g_free(header);
+	if (header->responseURI)
+		smlSafeCFree(&(header->responseURI));
+	
+	smlSafeFree((gpointer *)&header);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
-};
+}
 
 SmlItem *smlItemNew(unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%i, %p)", __func__, size, error);
+	CHECK_ERROR_REF
 		
 	SmlItem *item = smlTryMalloc0(sizeof(SmlItem), error);
 	if (!item)
@@ -332,6 +340,7 @@ error:
 SmlItem *smlItemNewForData(const char *data, unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, data, size, error);
+	CHECK_ERROR_REF
 	
 	SmlItem *item = smlItemNew(size, error);
 	if (!item)
@@ -369,7 +378,7 @@ void smlItemUnref(SmlItem *item)
 	smlAssert(item);
 	
 	if (g_atomic_int_dec_and_test(&(item->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (item->source)
 			smlLocationUnref(item->source);
@@ -377,6 +386,12 @@ void smlItemUnref(SmlItem *item)
 		if (item->target)
 			smlLocationUnref(item->target);
 		
+		if (item->sourceParent)
+			smlLocationUnref(item->sourceParent);
+			
+		if (item->targetParent)
+			smlLocationUnref(item->targetParent);
+		
 		if (item->anchor)
 			smlAnchorFree(item->anchor);
 		
@@ -384,10 +399,9 @@ void smlItemUnref(SmlItem *item)
 			xmlBufferFree(item->buffer);
 		
 		if (item->contenttype)
-			g_free(item->contenttype);
+			smlSafeCFree(&(item->contenttype));
 		
-		g_free(item);
-		item = NULL;
+		smlSafeFree((gpointer *)&item);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s: %i", __func__, item ? item->refCount : 0);
@@ -396,6 +410,7 @@ void smlItemUnref(SmlItem *item)
 SmlBool smlItemAddData(SmlItem *item, const char *data, unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, item, data, size, error);
+	CHECK_ERROR_REF
 	
 	if (item->size && xmlBufferLength(item->buffer) + size > item->size) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add data. size limit reached");
@@ -428,15 +443,16 @@ error:
 /** Checks if the item is complete */
 SmlBool smlItemCheck(SmlItem *item)
 {
+	smlAssert(xmlBufferLength(item->buffer) >= 0);
 	smlAssert(item);
 	if (!item->size)
 		return TRUE;
 		
-	if (xmlBufferLength(item->buffer) != item->size)
+	if ((unsigned int)xmlBufferLength(item->buffer) != item->size)
 	{
 		smlTrace(TRACE_INTERNAL, "%s: failed because size (%d != %d) does not match (%s).",
 			__func__, item->size,
-			xmlBufferLength(item->buffer), xmlBufferContent(item->buffer));
+			xmlBufferLength(item->buffer), VA_STRING((char *)xmlBufferContent(item->buffer)));
 		return FALSE;
 	}
 		
@@ -454,6 +470,7 @@ SmlBool smlItemHasData(SmlItem *item)
 SmlBool smlItemStealData(SmlItem *item, char **data, unsigned int *size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, item, data, size, error);
+	CHECK_ERROR_REF
 	smlAssert(size);
 	
 	if (!smlItemCheck(item)) {
@@ -462,7 +479,7 @@ SmlBool smlItemStealData(SmlItem *item, char **data, unsigned int *size, SmlErro
 	}
 	
 	*size = xmlBufferLength(item->buffer);
-	*data = g_strndup(xmlBufferContent(item->buffer), *size);
+	*data = g_strndup((const char *) xmlBufferContent(item->buffer), *size);
 	xmlBufferFree(item->buffer);
 	item->buffer = NULL;
 	
@@ -478,6 +495,7 @@ error:
 SmlBool smlItemGetData(SmlItem *item, char **data, unsigned int *size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, item, data, size, error);
+	CHECK_ERROR_REF
 	
 	if (!smlItemCheck(item)) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Item check failed");
@@ -527,6 +545,38 @@ SmlLocation *smlItemGetTarget(SmlItem *item)
 	return item->target;
 }
 
+void smlItemSetSourceParent(SmlItem *item, SmlLocation *sourceParent)
+{
+	smlAssert(item);
+	smlAssert(sourceParent);
+	
+	item->sourceParent = sourceParent;
+	smlLocationRef(sourceParent);
+}
+
+SmlLocation *smlItemGetSourceParent(SmlItem *item)
+{
+	smlAssert(item);
+	
+	return item->sourceParent;
+}
+
+void smlItemSetTargetParent(SmlItem *item, SmlLocation *targetParent)
+{
+	smlAssert(item);
+	smlAssert(targetParent);
+	
+	item->targetParent = targetParent;
+	smlLocationRef(targetParent);
+}
+
+SmlLocation *smlItemGetTargetParent(SmlItem *item)
+{
+	smlAssert(item);
+	
+	return item->targetParent;
+}
+
 void smlItemSetRaw(SmlItem *item, SmlBool raw)
 {
 	smlAssert(item);
@@ -539,7 +589,8 @@ SmlCred *smlCredNewFromString(const char *type,
 			 const char *data,
 			 SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %s, %s, %p)", __func__, data, type, format, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %s, %s, %p)", __func__, VA_STRING(data), VA_STRING(type), VA_STRING(format), error);
+	CHECK_ERROR_REF
 
 	SmlAuthType smlType = SML_AUTH_TYPE_UNKNOWN;
 	SmlFormatType smlFormat = SML_FORMAT_TYPE_UNKNOWN;
@@ -577,38 +628,37 @@ SmlCred *smlCredNewAuth(SmlAuthType type,
 			const char *password,
 			SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%d, %s, %p)", __func__, type, username, error);
-        char *data = NULL;
-	SmlFormatType format = SML_FORMAT_TYPE_BASE64;
+	smlTrace(TRACE_ENTRY, "%s(%d, %s, %p)", __func__, type, VA_STRING(username), error);
+	CHECK_ERROR_REF
 
-	// build plain data
-	if (type == SML_AUTH_TYPE_BASIC) {
-		char *plain = g_strjoin(":", username, password, NULL);
-		data = g_base64_encode(plain, strlen(plain));
-		g_free(plain);
-	} else if (type == SML_AUTH_TYPE_MD5) {
-		char *plain = g_strjoin(":", username, password, NULL);
-		unsigned char digest[16];
-		smlMD5GetDigest(plain, strlen(plain), digest);
-		g_free(plain);
-		data = smlMD5ToString(digest, NULL);
-		g_free(digest);
-	} else {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Unknown type - %s (%s).", type, __func__);
+	SmlCred *cred = NULL;
+
+	if (username ==  NULL || !strlen(username)) {
+		smlErrorSet(error, SML_ERROR_INTERNAL_MISCONFIGURATION,
+			"If authentication should be used then the username must be set.");
+		goto error;
+	}
+	if (password ==  NULL || !strlen(password)) {
+		smlErrorSet(error, SML_ERROR_INTERNAL_MISCONFIGURATION,
+			"If authentication should be used then the password must be set.");
 		goto error;
 	}
 
-	// the username is sometimes required as location name in the source
-	//     the username is readable from data     => OPTIONAL (e.g. syncml:auth-basic)
-	//     the username is not readable from data => REQUIRED (e.g. syncml:auth-md5)
-	SmlCred *cred = smlCredNew(type, format, data, username, error);
-	g_free(data);
+        cred = smlTryMalloc0(sizeof(SmlCred), error);
+        if (!cred)
+                goto error;
+
+	cred->refCount = 1;
+	cred->format = SML_FORMAT_TYPE_BASE64;
+	cred->type = type;
+	cred->username = g_strdup(username);
+	cred->password = g_strdup(password);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return cred;
 error:
-	if (data)
-		g_free(data);
+	if (cred)
+		smlSafeFree((gpointer *)&cred);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;	
 }
@@ -619,7 +669,8 @@ SmlCred *smlCredNew(SmlAuthType type,
 		const char *username,
 		SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %d, %d, %p)", __func__, data, type, format, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %d, %d, %p)", __func__, VA_STRING(data), type, format, error);
+	CHECK_ERROR_REF
 
         SmlCred *cred = smlTryMalloc0(sizeof(SmlCred), error);
         if (!cred)
@@ -629,25 +680,25 @@ SmlCred *smlCredNew(SmlAuthType type,
 	cred->format = format;
 	cred->data = g_strdup(data);
 	if (username)
-		cred->locName = g_strdup(username);
+		cred->username = g_strdup(username);
 	else
-		cred->locName = NULL;
+		cred->username = NULL;
 	cred->refCount = 1;
 
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, cred);
 	return cred;
 error:
 	if (cred->data)
-		g_free(cred->data);
-	if (cred->locName)
-		g_free(cred->locName);
+		smlSafeCFree(&(cred->data));
+	if (cred->username)
+		smlSafeCFree(&(cred->username));
 	if (cred)
-		g_free(cred);
+		smlSafeFree((gpointer *)&cred);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
 }
 
-SmlCred *smlCredRef(SmlCred *cred)
+void smlCredRef(SmlCred *cred)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, cred);
 	smlAssert(cred);
@@ -655,7 +706,6 @@ SmlCred *smlCredRef(SmlCred *cred)
 	g_atomic_int_inc(&(cred->refCount));
 	
 	smlTrace(TRACE_EXIT, "%s: New refcount: %i", __func__, cred->refCount);
-	return cred;
 }
 
 void smlCredUnref(SmlCred *cred)
@@ -664,14 +714,16 @@ void smlCredUnref(SmlCred *cred)
 	smlAssert(cred);
 	
 	if (g_atomic_int_dec_and_test(&(cred->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (cred->data)
-			g_free(cred->data);
-		if (cred->locName)
-			g_free(cred->locName);
+			smlSafeCFree(&(cred->data));
+		if (cred->username)
+			smlSafeCFree(&(cred->username));
+		if (cred->password)
+			smlSafeCFree(&(cred->password));
 			
-		g_free(cred);
+		smlSafeFree((gpointer *)&cred);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -680,25 +732,176 @@ void smlCredUnref(SmlCred *cred)
 /* FIXME: DEPRECATED */
 void smlCredFree(SmlCred *cred)
 {
-	return smlCredUnref(cred);
+	smlCredUnref(cred);
 }
 
-void smlChalFree(SmlChal *chal)
+SmlChal *smlChalNew(SmlAuthType type, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, chal);
+	smlTrace(TRACE_ENTRY, "%s(%u, %p)", __func__, type, error);
+	CHECK_ERROR_REF
+	SmlChal *chal = NULL;
+
+	/* allocate memory */
+	smlAssert(type != SML_AUTH_TYPE_UNKNOWN);
+	chal = smlTryMalloc0(sizeof(SmlChal), error);
+	if (!chal)
+		goto error;
+	chal->refCount = 1;
+	chal->type = type;
+	chal->format = SML_FORMAT_TYPE_BASE64;
+
+	if (type == SML_AUTH_TYPE_MD5)
+	{
+		/* A nonce must be generated for this type.
+		 *     minimum strength:  2^128
+		 *     strength per byte: 2^6 - 2 > 2^5
+		 *     needed bytes:      128 / 5 < 130 / 5 = 26
+		 */
+		chal->nonce_plain = smlRandStr(26, TRUE);
+		chal->nonce_length = 26;
+		chal->nonce_b64 = g_base64_encode(
+						(const unsigned char *) chal->nonce_plain,
+						chal->nonce_length);
+		if (!chal->nonce_b64) {
+			smlErrorSet(error, SML_ERROR_GENERIC,
+				"The nonce of the challenge cannot be base64 encoded.");
+			goto error;
+		}
+	}
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return chal;
+error:
+	if (chal->nonce_plain)
+		smlSafeCFree(&(chal->nonce_plain));
+	if (chal->nonce_b64)
+		smlSafeCFree(&(chal->nonce_b64));
+	if (chal)
+		smlSafeFree((gpointer *)&chal);
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+	return NULL;
+}
+
+SmlChal *smlChalNewFromBinary(
+		SmlAuthType type,
+		const char *nonce,
+		size_t length,
+		SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s", __func__);
+	CHECK_ERROR_REF
+	SmlChal *chal = NULL;
+
+	/* only syncml:auth-md5 needs a nonce */
+	smlAssert(type == SML_AUTH_TYPE_MD5);
+
+	/* allocate memory */
+	chal = smlTryMalloc0(sizeof(SmlChal), error);
+	if (!chal)
+		goto error;
+	chal->refCount = 1;
+
+	/* copy nonce */
+	chal->type = SML_AUTH_TYPE_MD5;
+	chal->format = SML_FORMAT_TYPE_BASE64;
+	chal->nonce_plain = g_strndup(nonce, length);
+	chal->nonce_length = length;
+
+	/* create base64 nonce */
+	chal->nonce_b64 = g_base64_encode((const unsigned char *) nonce, length);
+	if (!chal->nonce_b64) {
+		smlErrorSet(error, SML_ERROR_GENERIC,
+			"The base64 encoding of the nonce failed.");
+		goto error;
+	}
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return chal;
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+	return NULL;
+}
+
+SmlChal *smlChalNewFromBase64(
+		SmlAuthType type,
+		const char *nonce,
+		SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s", __func__);
+	CHECK_ERROR_REF
+	SmlChal *chal = NULL;
+
+	/* only syncml:auth-md5 needs a nonce */
+	smlAssert(type == SML_AUTH_TYPE_MD5);
+
+	/* allocate memory */
+	chal = smlTryMalloc0(sizeof(SmlChal), error);
+	if (!chal)
+		goto error;
+	chal->refCount = 1;
+
+	/* copy nonce */
+	chal->type = SML_AUTH_TYPE_MD5;
+	chal->format = SML_FORMAT_TYPE_BASE64;
+	chal->nonce_b64 = g_strdup(nonce);
+
+	/* create binary nonce */
+	size_t length = 0;
+	chal->nonce_plain = (char *) g_base64_decode(nonce, &length);
+	if (!chal->nonce_plain || length < 1) {
+		smlErrorSet(error, SML_ERROR_GENERIC,
+			"The base64 encoded nonce cannot be decoded.");
+		goto error;
+	}
+	chal->nonce_length = length;
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return chal;
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(error));
+	return NULL;
+}
+
+void smlChalRef(SmlChal *chal)
+{
+	smlTrace(TRACE_ENTRY, "%s", __func__);
 	smlAssert(chal);
 	
-	if (chal->nonce)
-		g_free(chal->nonce);
+	g_atomic_int_inc(&(chal->refCount));
 	
-	g_free(chal);
+	smlTrace(TRACE_EXIT, "%s: New refcount: %i", __func__, chal->refCount);
+}
+
+void smlChalUnref(SmlChal *chal)
+{
+	smlTrace(TRACE_ENTRY, "%s", __func__);
+	smlAssert(chal);
+	
+	if (g_atomic_int_dec_and_test(&(chal->refCount))) {
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
+		
+		if (chal->nonce_plain)
+			smlSafeCFree(&(chal->nonce_plain));
+
+		if (chal->nonce_b64)
+			smlSafeCFree(&(chal->nonce_b64));
+
+		smlSafeFree((gpointer *)&chal);
+	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
+/* FIXME: DEPRECATED */
+void smlChalFree(SmlChal *chal)
+{
+	smlChalUnref(chal);
+}
+
 SmlMapItem *smlMapItemNew(const char *uid, const char *newuid, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, uid, newuid, error);
+	smlTrace(TRACE_ENTRY, "%s(%s, %s, %p)", __func__, VA_STRING(uid), VA_STRING(newuid), error);
+	CHECK_ERROR_REF
 	smlAssert(uid);
 	smlAssert(newuid);
 	
@@ -721,7 +924,7 @@ SmlMapItem *smlMapItemNew(const char *uid, const char *newuid, SmlError **error)
 error_free_source:
 	smlLocationUnref(item->source);
 error_free_item:
-	g_free(item);
+	smlSafeFree((gpointer *)&item);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;	
@@ -744,18 +947,23 @@ void smlMapItemUnref(SmlMapItem *item)
 	smlAssert(item);
 	
 	if (g_atomic_int_dec_and_test(&(item->refCount))) {
-		smlTrace(TRACE_INTERNAL, "Refcount == 0!");
+		smlTrace(TRACE_INTERNAL, "%s: Refcount == 0!", __func__);
 		
 		if (item->source)
+		{
 			smlLocationUnref(item->source);
+			item->source = NULL;
+		}
 		
 		if (item->target)
+		{
 			smlLocationUnref(item->target);
+			item->target = NULL;
+		}
 			
-		g_free(item);
+		smlSafeFree((gpointer *)&item);
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-/*@}*/
