@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
 
+/**
+ * @defgroup SmlElements SyncML Elements
+ * @ingroup PublicLowLevelAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_ELEMENTS_H_
 #define _SML_ELEMENTS_H_
 
@@ -36,7 +45,7 @@ SmlBool		smlLocationIsRelative	(SmlLocation *location);
 SmlCred *smlCredNewFromString(const char *type, const char *format, const char *data, SmlError **error);
 SmlCred *smlCredNewAuth(SmlAuthType type, const char *username, const char *password, SmlError **error);
 SmlCred *smlCredNew(SmlAuthType type, SmlFormatType format, const char *data, const char*username, SmlError **error);
-SmlCred *smlCredRef(SmlCred *cred);
+void smlCredRef(SmlCred *cred);
 void smlCredUnref(SmlCred *cred);
 void smlCredFree(SmlCred *cred) LIBSYNCML_DEPRECATED; /* expire date: 20090120 */
 
@@ -57,14 +66,25 @@ void smlItemSetSource(SmlItem *item, SmlLocation *source);
 SmlLocation *smlItemGetSource(SmlItem *item);
 void smlItemSetTarget(SmlItem *item, SmlLocation *target);
 SmlLocation *smlItemGetTarget(SmlItem *item);
+void smlItemSetSourceParent(SmlItem *item, SmlLocation *sourceParent);
+SmlLocation *smlItemGetSourceParent(SmlItem *item);
+void smlItemSetTargetParent(SmlItem *item, SmlLocation *targeParent);
+SmlLocation *smlItemGetTargetParent(SmlItem *item);
 void smlItemSetRaw(SmlItem *item, SmlBool raw);
 
 void smlHeaderFree(SmlHeader *header);
 
-void smlChalFree(SmlChal *chal);
+SmlChal *smlChalNew(SmlAuthType type, SmlError **error);
+SmlChal *smlChalNewFromBinary(SmlAuthType type, const char *nonce, size_t length, SmlError **error);
+SmlChal *smlChalNewFromBase64(SmlAuthType type, const char *nonce, SmlError **error);
+void smlChalRef(SmlChal *chal);
+void smlChalUnref(SmlChal *chal);
+void smlChalFree(SmlChal *chal) LIBSYNCML_DEPRECATED; /* expire date: 20090417 */
 
 SmlMapItem *smlMapItemNew(const char *uid, const char *newuid, SmlError **error);
 SmlMapItem *smlMapItemRef(SmlMapItem *item);
 void smlMapItemUnref(SmlMapItem *item);
 
 #endif //_SML_ELEMENTS_H_
+
+/*@}*/
