@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2009  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,35 +25,15 @@
 #include "sml_notification_internals.h"
 #include "sml_transport_internals.h"
 #include "sml_command_internals.h"
-
-/**
- * @defgroup GroupIDPrivate Group Description Internals
- * @ingroup ParentGroupID
- * @brief The private part
- * 
- */
-/*@{*/
-
-/*@}*/
-
-/**
- * @defgroup GroupID Group Description
- * @ingroup ParentGroupID
- * @brief What does this group do?
- * 
- */
-/*@{*/
-
-/** @brief Function description
- * 
- * @param parameter This parameter does great things
- * @returns What you always wanted to know
- * 
- */
+#include "sml_manager_internals.h"
+#include "sml_error_internals.h"
+#include "objects/sml_auth_internals.h"
+#include "sml_elements_internals.h"
 
 SmlNotification *smlNotificationNew(SmlNotificationVersion version, SmlNotificationUIMode mode, SmlNotificationInitiator init, unsigned int sessionID, const char *identifier, const char *target, SmlMimeType type, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%i, %i, %i, %i, %s, %s, %i, %p)", __func__, version, mode, init, sessionID, identifier, target, type, error);
+	smlTrace(TRACE_ENTRY, "%s(%i, %i, %i, %i, %s, %s, %i, %p)", __func__, version, mode, init, sessionID, VA_STRING(identifier), VA_STRING(target), type, error);
+	CHECK_ERROR_REF
 
 	SmlNotification *san = smlTryMalloc0(sizeof(SmlNotification), error);
 	if (!san)
@@ -65,8 +46,6 @@ SmlNotification *smlNotificationNew(SmlNotificationVersion version, SmlNotificat
 	san->identifier = g_strdup(identifier);
 	san->type = type;
 	san->cred = NULL;
-	// FIXME: this is stupid default but it is required
-	// FIXME: because the original code only supports servers
 	san->sessionType = SML_SESSION_TYPE_SERVER;
 	san->manager = NULL;
 	
@@ -96,6 +75,7 @@ void smlNotificationSetCred(SmlNotification *san, SmlCred *cred)
 	// if we set no credentials then we always assume a server
 	// FIXME: this is a hack because the original implementation
 	// FIXME: only support server
+	// Only one question which client sends a notification!?
 	san->sessionType = SML_SESSION_TYPE_CLIENT;
 	smlCredRef(cred);
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -121,13 +101,19 @@ void smlNotificationFree(SmlNotification *san)
 		smlNotificationFreeAlert(alert);
 	}
 	
-	if (san->target)
+	if (san->target) {
 		smlLocationUnref(san->target);
+		san->target = NULL;
+	}
 	
-	g_free(san->identifier);
-	if (san->cred)
+	if (san->cred) {
 		smlCredUnref(san->cred);
-	g_free(san);
+		san->cred = NULL;
+	}
+
+	if (san->identifier)
+		smlSafeCFree(&(san->identifier));
+	smlSafeFree((gpointer *)&san);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -135,6 +121,7 @@ void smlNotificationFree(SmlNotification *san)
 SmlBool smlNotificationNewAlert(SmlNotification *san, SmlAlertType type, const char *contenttype, const char *serverURI, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, error);
+	CHECK_ERROR_REF
 	smlAssert(type >= 206 && type <= 210);
 
 	SmlSanAlert *alert = smlTryMalloc0(sizeof(SmlSanAlert), error);
@@ -160,9 +147,9 @@ void smlNotificationFreeAlert(SmlSanAlert *alert)
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, alert);
 	smlAssert(alert);
 	
-	g_free(alert->contenttype);
-	g_free(alert->serverURI);
-	g_free(alert);
+	smlSafeCFree(&(alert->contenttype));
+	smlSafeCFree(&(alert->serverURI));
+	smlSafeFree((gpointer *)&alert);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -170,9 +157,9 @@ void smlNotificationFreeAlert(SmlSanAlert *alert)
 SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, data, size, error);
+	CHECK_ERROR_REF
 	smlAssert(data);
 	smlAssert(size);
-	SmlSanAlert *alert = NULL;
 	
 	if (size < 25) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Size too small");
@@ -188,6 +175,7 @@ SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlEr
 	SmlNotification *san = smlTryMalloc0(sizeof(SmlNotification), error);
 	if (!san)
 		goto error;
+	san->type = SML_MIMETYPE_SAN;
 
 	/* Version */
 	unsigned int version = ((uint8_t)data[16]) << 2;
@@ -213,7 +201,8 @@ SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlEr
 	
 	/* server identifier */
 	if (idLength) {
-		san->identifier = smlTryMalloc0(idLength, error);
+		/* the trailing NULL byte is usually not present in the SAN */
+		san->identifier = smlTryMalloc0(idLength + 1, error);
 		if (!san->identifier)
 			goto error_free_san;
 	
@@ -224,7 +213,7 @@ SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlEr
 	data += 25 + idLength;
 	unsigned int alertLength = 25 + idLength;
 
-	int i = 0;
+	unsigned int i = 0;
 	for (i = 0; i < numSync; i++) {
 		if (size < alertLength + 5) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Size too small3");
@@ -238,14 +227,9 @@ SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlEr
 		}
 		alertLength = alertLength + 5 + idLength;
 		
-		alert = smlTryMalloc0(sizeof(SmlSanAlert), error);
-		if (!alert)
-			goto error_free_san;
-		san->alerts = g_list_append(san->alerts, alert);
-		
 		/* Alert type + future */
-		alert->type = (((uint8_t)data[0]) >> 4) + 200;
-		if (alert->type < 206 || alert->type > 210) {
+		SmlAlertType alert_type = (((uint8_t)data[0]) >> 4) + 200;
+		if (alert_type < 206 || alert_type > 210) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Wrong alert type");
 			goto error_free_san;
 		}
@@ -255,30 +239,64 @@ SmlNotification *smlNotificationParse(const char *data, unsigned int size, SmlEr
 		contenttype = contenttype | ((uint8_t)data[3]);
 		
 		/* The contenttype */
+		const char *alert_ct;
 		switch (contenttype) {
+			case 0x00:
+				/* This is exactly like defined in "DS Protocol" section
+				 * 12.4.13 "Content Type" of the official OMA document
+				 * OMA-TS-DS_Protocol-V1_2_1-20070810-A.pdf.
+				 */ 
+				alert_ct = NULL;
+				break;
 			case 0x07:
-				alert->contenttype = g_strdup(SML_ELEMENT_TEXT_VCARD);
+				alert_ct = SML_ELEMENT_TEXT_VCARD;
 				break;
 			case 0x06:
-				alert->contenttype = g_strdup(SML_ELEMENT_TEXT_VCAL);
+				alert_ct = SML_ELEMENT_TEXT_VCAL;
 				break;
 			case 0x03:
-				alert->contenttype = g_strdup(SML_ELEMENT_TEXT_PLAIN);
+				alert_ct = SML_ELEMENT_TEXT_PLAIN;
+				break;
+			case 0x0305:
+				alert_ct = SML_ELEMENT_TEXT_ICAL;
+				break;
+			case 0x0306:
+				alert_ct = SML_ELEMENT_APPLICATION_OMA_DS_EMAIL;
+				break;
+			case 0x0307:
+				alert_ct = SML_ELEMENT_APPLICATION_OMA_DS_FILE;
+				break;
+			case 0x0308:
+				alert_ct = SML_ELEMENT_APPLICATION_OMA_DS_FOLDER;
+				break;
+			case 0x0309:
+				alert_ct = SML_ELEMENT_TEXT_VCARD_30;
 				break;
 			default:
-				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown content type");
+				/* http://www.openmobilealliance.org/Tech/omna/omna-wsp-content-type.aspx */
+				smlErrorSet(error, SML_ERROR_GENERIC, "The OMNA WSP Content Type Number 0x%xd is not supported.", contenttype);
 				goto error_free_san;
 		}
 		
 		/* The server uri */
+		char *alert_uri;
 		if (idLength) {
-			alert->serverURI = smlTryMalloc0(idLength, error);
-			if (!alert->serverURI)
+			/* the trailing NULL byte is usually not present in the SAN */
+			alert_uri = smlTryMalloc0(idLength + 1, error);
+			if (!alert_uri)
 				goto error_free_san;
-				
-			memcpy(alert->serverURI, data + 5, idLength);
+
+			memcpy(alert_uri, data + 5, idLength);
 		}
 		data += 5 + idLength;
+
+		/* Create new alert */
+		if (!smlNotificationNewAlert(san, alert_type, alert_ct, alert_uri, error))
+			goto error_free_san;
+
+		/* Cleanup */
+		if (idLength)
+			smlSafeCFree(&alert_uri);
 	}
 
 	if (size > alertLength) {
@@ -294,46 +312,57 @@ error_free_san:
 		smlNotificationFreeAlert(san->alerts->data);
 		san->alerts = g_list_delete_link(san->alerts, san->alerts);
 	}
-	g_free(san->identifier);
-	g_free(san);
+	if (san->identifier)
+		smlSafeCFree(&(san->identifier));
+	smlSafeFree((gpointer *)&san);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
 }
 
-static SmlBool _smlNotificationAssemble11(SmlNotification *san, char **data, unsigned int *size, SmlProtocolVersion version, SmlError **error)
+SmlBool smlNotificationAssemble11(SmlNotification *san, char **data, unsigned int *size, SmlProtocolVersion version, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %p)", __func__, san, data, size, version, error);
+	CHECK_ERROR_REF
 	smlAssert(san);
 	smlAssert(data);
 	smlAssert(size);
+
+	/* If a notification is sent via a normal SyncML message
+	 * then the used session must be managed to guarantee the
+	 * consistency of the session.
+	 */
+	smlAssert(san->manager);
+
+	SmlAssembler *assm = NULL;
+	SmlLocation *source = NULL;
+	SmlSession *session = NULL;
 	SmlSanAlert *alert = NULL;
 	SmlCommand *cmd = NULL;
 
-	SmlAssembler *assm = smlAssemblerNew(san->type, 0, error);
+	source = smlLocationNew(san->identifier, NULL, error);
+	if (!source)
+		goto error;
+
+	char *sessionString = smlManagerGetNewSessionID(san->manager);
+	session = smlSessionNew(san->sessionType, san->type, version, SML_PROTOCOL_SYNCML, san->target, source, sessionString, 0, error);
+	if (!session) {
+		smlSafeCFree(&sessionString);
+		smlLocationUnref(source);
+		goto error;
+	}
+	smlSafeCFree(&sessionString);
+	smlManagerSessionAdd(san->manager, session, NULL, error);
+
+	assm = smlAssemblerNew(san->type, 0, error);
 	if (!assm)
 		goto error;
 	
-	SmlLocation *source = smlLocationNew(san->identifier, NULL, error);
-	if (!source)
-		goto error_free_assm;
-
-	SmlSession *session = smlSessionNew(san->sessionType, san->type, version, SML_PROTOCOL_SYNCML, san->target, source, "1", 0, error);
-	if (!session) {
-		smlLocationUnref(source);
-		goto error_free_assm;
-	}
-	if (san->cred)
-		smlSessionSetCred(session, san->cred);
-	if (san->manager)
-		smlManagerSessionAdd(san->manager, session, NULL, error);
-
 	if (!smlAssemblerStart(assm, session, error))
-		goto error_free_session;
-	
+		goto error;
 	
 	if (!smlAssemblerAddHeader(assm, session, error))
-		goto error_free_session;
+		goto error;
 	
 	GList *a = NULL;
 	int i = 1;
@@ -343,22 +372,22 @@ static SmlBool _smlNotificationAssemble11(SmlNotification *san, char **data, uns
 		
 		SmlLocation *loc = smlLocationNew(alert->serverURI, NULL, error);
 		if (!loc)
-			goto error_free_session;
+			goto error;
 		
 		cmd = smlCommandNewAlert(SML_ALERT_TWO_WAY_BY_SERVER, NULL, loc, NULL, NULL, alert->contenttype, error);
 		if (!cmd) {
 			smlLocationUnref(loc);
-			goto error_free_session;
+			goto error;
 		}
 		
 		cmd->cmdID = i;
 		i++;
 		
 		if (!smlAssemblerStartCommand(assm, NULL, cmd, error) == SML_ASSEMBLER_RESULT_OK)
-			goto error_free_cmd;
+			goto error;
 			
 		if (!smlAssemblerEndCommand(assm, NULL, error))
-			goto error_free_cmd;
+			goto error;
 			
 		smlLocationUnref(loc);
 		
@@ -366,7 +395,7 @@ static SmlBool _smlNotificationAssemble11(SmlNotification *san, char **data, uns
 	}
 	
 	if (!smlAssemblerRun(assm, data, size, NULL, TRUE, error))
-		goto error_free_session;
+		goto error;
 	
 	smlAssemblerFlush(assm);
 	
@@ -378,18 +407,18 @@ static SmlBool _smlNotificationAssemble11(SmlNotification *san, char **data, uns
 	
 	char *hex = smlPrintHex(*data, *size);
 	smlTrace(TRACE_INTERNAL, "San packet assembled: %s", hex);
-	g_free(hex);
+	smlSafeCFree(&hex);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 	
-error_free_cmd:
-	smlCommandUnref(cmd);
-error_free_session:
-	smlSessionUnref(session);
-error_free_assm:
-	smlAssemblerFree(assm);
 error:
+	if (cmd)
+		smlCommandUnref(cmd);
+	if (session)
+		smlSessionUnref(session);
+	if (assm)
+		smlAssemblerFree(assm);
 	*data = NULL;
 	*size = 0;
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
@@ -399,18 +428,20 @@ error:
 static SmlBool _smlNotificationAssemble12(SmlNotification *san, char **data, unsigned int *size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, san, data, size, error);
+	CHECK_ERROR_REF
 	smlAssert(san);
 	smlAssert(data);
+	smlAssert(*data == NULL);
 	smlAssert(size);
 	unsigned int numsync = 0;
 	SmlSanAlert *alert = NULL;
 	
 	/* Calculate the length of the san data */
-	unsigned int length = 16 /* Digest */ + 8 /* Hdr */ + strlen(san->identifier) + 1 /* Identifier */ + 1 /* num-sync + future */;	
+	unsigned int length = 16 /* Digest */ + 8 /* Hdr */ + strlen(san->identifier) /* Identifier */ + 1 /* num-sync + future */;
 	GList *a = NULL;
 	for (a = san->alerts; a; a = a->next) {
 		alert = a->data;
-		length += 5 + strlen(alert->serverURI) + 1;
+		length += 5 + strlen(alert->serverURI);
 		numsync++;
 	}
 	
@@ -420,11 +451,11 @@ static SmlBool _smlNotificationAssemble12(SmlNotification *san, char **data, uns
 		goto error;
 	*data = buffer;
 	*size = length;
-	
-	/* FIXME: Now create the digest */
+
+	/* the digest must be created at the end */
 	
 	/* Create Header */
-	/* Version */
+	/* Version - 10 bit */
 	buffer[16] = 0x03;
 
 	/* ui mode */
@@ -433,17 +464,17 @@ static SmlBool _smlNotificationAssemble12(SmlNotification *san, char **data, uns
 	/* initiator */
 	buffer[17] = buffer[17] | (san->init << 3);
 	
-	/* future use is 0 for now */
+	/* future use is 0 for now - 27 bit */
 
 	/* session id */
 	buffer[21] = san->sessionID >> 8;
 	buffer[22] = san->sessionID;
 	
-	/* server identifier */
-	buffer[23] = strlen(san->identifier) + 1;
-	strcpy(buffer + 24, san->identifier);
+	/* server identifier without trailing NULL byte */
+	buffer[23] = strlen(san->identifier);
+	strncpy(buffer + 24, san->identifier, strlen(san->identifier));
 	
-	buffer += 24 + strlen(san->identifier) + 1;
+	buffer += 24 + strlen(san->identifier);
 	
 	/* Num syncs + future */
 	buffer[0] = numsync << 4;
@@ -455,33 +486,90 @@ static SmlBool _smlNotificationAssemble12(SmlNotification *san, char **data, uns
 		buffer[0] = (alert->type - 200) << 4;
 		
 		/* The contenttype */
-		if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_VCARD))
+		if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_VCARD)) {
 			buffer[3] = 0x07;
-		else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_VCAL))
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_VCAL)) {
 			buffer[3] = 0x06;
-		else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_PLAIN))
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_PLAIN)) {
 			buffer[3] = 0x03;
-		else {
-			smlErrorSet(error, SML_ERROR_GENERIC, "Unknown content type");
-			goto error_free_data;
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_ICAL)) {
+			buffer[2] = 0x03;
+			buffer[3] = 0x05;
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_APPLICATION_OMA_DS_EMAIL)) {
+			buffer[2] = 0x03;
+			buffer[3] = 0x06;
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_APPLICATION_OMA_DS_FILE)) {
+			buffer[2] = 0x03;
+			buffer[3] = 0x07;
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_APPLICATION_OMA_DS_FOLDER)) {
+			buffer[2] = 0x03;
+			buffer[3] = 0x08;
+		} else if (!strcmp(alert->contenttype, SML_ELEMENT_TEXT_VCARD_30)) {
+			buffer[2] = 0x03;
+			buffer[3] = 0x09;
+		} else {
+			/* http://www.openmobilealliance.org/Tech/omna/omna-wsp-content-type.aspx */
+			smlTrace(TRACE_INTERNAL,
+				"%s: The OMNA WSP Content Type Number is unknown for %s.",
+				__func__, alert->contenttype);
+			/* All bytes are already set to zero. */
 		}
 		
-		/* The server uri */
-		buffer[4] = strlen(alert->serverURI) + 1;
-		strcpy(buffer + 5, alert->serverURI);
-		buffer += 5 + strlen(alert->serverURI) + 1;
+		/* The server uri without a trailing NULL byte */
+		buffer[4] = strlen(alert->serverURI);
+		strncpy(buffer + 5, alert->serverURI, strlen(alert->serverURI));
+		buffer += 5 + strlen(alert->serverURI);
 	}
+
+	/* Create the digest */
+
+	/* DO NOT USE buffer because this pointer was manipulated
+	 * ALWAYS USE *data.
+	 */
+
+	/* base64 encoded MD5 digest of the notification */
+	unsigned char digest[16];
+	smlMD5GetDigest (*data+16, length - 16 /* digest */, digest);
+	char *b64_san = g_base64_encode(digest, 16);
+	smlTrace(TRACE_INTERNAL, "%s: b64(md5(san)) ::= %s", __func__, b64_san);
+
+	/* base64 encoded MD5 digest of username:password */
+	const char *username = "";
+	const char *password = "";
+	if (san->cred) {
+		username = san->cred->username;
+		password = san->cred->password;
+	}
+	char *auth = g_strdup_printf("%s:%s", username, password);
+	smlMD5GetDigest (auth, strlen(auth), digest);
+	smlSafeCFree(&auth);
+	char *b64_auth = g_base64_encode(digest, 16);
+	smlTrace(TRACE_INTERNAL, "%s: b64(md5(username:password)) ::= %s", __func__, b64_auth);
+
+	/* MD5 digest of auth:nonce:notification (empty nonce) */
+	char *complete = g_strdup_printf("%s::%s", b64_auth, b64_san);
+	smlSafeCFree(&b64_auth);
+	smlSafeCFree(&b64_san);
+	smlMD5GetDigest (complete, strlen(complete), digest);
+	smlSafeCFree(&complete);
+	smlTrace(TRACE_INTERNAL, "%s: md5(auth::san) complete", __func__);
+
+	/* copy digest to buffer */
+	memcpy(*data, digest, 16);
+	smlTrace(TRACE_INTERNAL, "%s: md5(auth::san) copied to san", __func__);
+
+	/* debug */
+
 	char *hex = smlPrintHex(*data, *size);
 	smlTrace(TRACE_INTERNAL, "San packet assembled: %s", hex);
-	g_free(hex);
+	smlSafeCFree(&hex);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_free_data:
-	g_free(*data);
 error:
-	*data = NULL;
+	if (*data)
+		smlSafeCFree(data);
 	*size = 0;
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
@@ -489,13 +577,14 @@ error:
 
 SmlBool smlNotificationAssemble(SmlNotification *san, char **data, unsigned int *size, SmlError **error)
 {
+	CHECK_ERROR_REF
 	switch (san->version) {
 		case SML_SAN_VERSION_12:
 			return _smlNotificationAssemble12(san, data, size, error);
 		case SML_SAN_VERSION_10:
-			return _smlNotificationAssemble11(san, data, size, SML_VERSION_10, error);
+			return smlNotificationAssemble11(san, data, size, SML_VERSION_10, error);
 		case SML_SAN_VERSION_11:
-			return _smlNotificationAssemble11(san, data, size, SML_VERSION_11, error);
+			return smlNotificationAssemble11(san, data, size, SML_VERSION_11, error);
 		case SML_SAN_VERSION_UNKNOWN:
 			smlErrorSet(error, SML_ERROR_GENERIC, "Unknown SAN version.");
 			break;
@@ -507,6 +596,7 @@ SmlBool smlNotificationAssemble(SmlNotification *san, char **data, unsigned int
 SmlBool smlNotificationSend(SmlNotification *san, SmlTransport *tsp, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, san, tsp, error);
+	CHECK_ERROR_REF
 	smlAssert(san);
 	smlAssert(tsp);
 
@@ -523,6 +613,7 @@ SmlBool smlNotificationSend(SmlNotification *san, SmlTransport *tsp, SmlError **
 	
 	if (!tspdata)
 		goto error_free_data;
+	/* Do not free data because smlTransportDateNew does copy by reference. */
 	data = NULL;
 	
 	/* For SyncML 1.2 and OBEX Transport we use SML_MIMETYPE_SAN for PUT and use the regular
@@ -535,7 +626,6 @@ SmlBool smlNotificationSend(SmlNotification *san, SmlTransport *tsp, SmlError **
 		goto error_free_tspdata;
 
 	smlTransportDataDeref(tspdata);
-	g_free(data);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -543,7 +633,7 @@ SmlBool smlNotificationSend(SmlNotification *san, SmlTransport *tsp, SmlError **
 error_free_tspdata:
 	smlTransportDataDeref(tspdata);
 error_free_data:
-	g_free(data);
+	smlSafeCFree(&data);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
@@ -607,5 +697,3 @@ const char *smlSanAlertGetServerURI(SmlSanAlert *alert)
 	smlAssert(alert);
 	return alert->serverURI;
 }
-
-/*@}*/
