@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,14 @@
  *
  */
 
+/**
+ * @defgroup SmlNotificationPrivate SyncML Notifications Internals
+ * @ingroup PrivateLowLevelAPI
+ * @brief The private part
+ * 
+ */
+/*@{*/
+
 #ifndef _SML_NOTIFICATION_INTERNALS_H_
 #define _SML_NOTIFICATION_INTERNALS_H_
 
@@ -33,6 +42,9 @@ struct SmlNotification {
 	SmlCred *cred;
 	SmlSessionType sessionType;
 	SmlManager *manager;
+
+	unsigned int localMaxMsgSize;
+	unsigned int localMaxObjSize;
 };
 
 struct SmlSanAlert {
@@ -41,4 +53,13 @@ struct SmlSanAlert {
 	char *serverURI;
 };
 
+SmlBool smlNotificationAssemble11(
+	SmlNotification *san,
+	char **data,
+	unsigned int *size,
+	SmlProtocolVersion version,
+	SmlError **error);
+
 #endif //_SML_NOTIFICATION_INTERNALS_H_
+
+/*@{*/
