@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,17 +22,19 @@
 #include "syncml.h"
 
 #include "syncml_internals.h"
+#include "sml_error_internals.h"
 #include "sml_parse_internals.h"
 #include "sml_command_internals.h"
 #include "sml_elements_internals.h"
+#include "sml_notification_internals.h"
 
 #include "parser/sml_xml_assm.h"
 #include "parser/sml_xml_parse.h"
 #include "parser/sml_wbxml.h"
 
 /**
- * @defgroup Parser SyncML Parser
- * @ingroup PublicAPI
+ * @defgroup SmlParser SyncML Parser
+ * @ingroup PublicLowLevelAPI
  * @brief Interfaces to parse syncml messages
  * 
  */
@@ -47,6 +50,7 @@
 SmlParser *smlParserNew(SmlMimeType type, unsigned int limit, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%i, %i, %p)", __func__, type, limit, error);
+	CHECK_ERROR_REF
 	
 	SmlParser *parser = smlTryMalloc0(sizeof(SmlParser), error);
 	if (!parser)
@@ -60,6 +64,11 @@ SmlParser *smlParserNew(SmlMimeType type, unsigned int limit, SmlError **error)
 			if (!(parser->parser_userdata = smlXmlParserNew(&(parser->functions), error)))
 				goto error_free_parser;
 			break;
+		case SML_MIMETYPE_SAN:
+			/* SANs will be converted to normal SyncML 1.1 notifications */
+			if (!(parser->parser_userdata = smlXmlParserNew(&(parser->functions), error)))
+				goto error_free_parser;
+			break;
 		case SML_MIMETYPE_WBXML:
 #ifdef ENABLE_WBXML
 			if (!(parser->parser_userdata = smlWbxmlParserNew(&(parser->functions), error)))
@@ -78,12 +87,19 @@ SmlParser *smlParserNew(SmlMimeType type, unsigned int limit, SmlError **error)
 	return parser;
 
 error_free_parser:
-	g_free(parser);
+	smlSafeFree((gpointer *)&parser);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
 }
 
+void smlParserSetManager(SmlParser *parser, SmlManager *manager)
+{
+	smlAssert(parser);
+	smlAssert(manager);
+	parser->manager = manager;
+}
+
 /** @brief Frees a parser
  * 
  * @param parser The parser to free
@@ -97,7 +113,7 @@ void smlParserFree(SmlParser *parser)
 	if (parser->functions.free)
 		parser->functions.free(parser->parser_userdata);
 	
-	g_free(parser);
+	smlSafeFree((gpointer *)&parser);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -116,18 +132,63 @@ void smlParserFree(SmlParser *parser)
 SmlBool smlParserStart(SmlParser *parser, const char *data, unsigned int size, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, parser, data, size, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(data);
 	smlAssert(size);
 	smlAssert(parser->functions.start);
-		
+	smlAssertMsg(parser->type != SML_MIMETYPE_SAN || parser->manager,
+		SML_ASSERT_MSG_SAN_PARSE_REQUIRES_MANAGER);
+	
 	if (parser->limit && size > parser->limit) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Input data too large");
 		goto error;
 	}
-	
-	if (!parser->functions.start(parser->parser_userdata, data, size, error))
-		goto error;
+
+	if (parser->type == SML_MIMETYPE_SAN) {
+		/* convert OMA DS 1.2 SAN into an SyncML 1.1 notification */
+		smlTrace(TRACE_INTERNAL, "%s - detected an OMA DS 1.2 SAN", __func__);
+
+		/* parse the SAN */
+		SmlNotification *san = smlNotificationParse(data, size, error);
+		if (!san)
+			goto error;
+
+		/* fix 1.2 SAN for use with SyncML 1.1 */
+		smlNotificationSetManager(san, parser->manager);
+		san->type = SML_MIMETYPE_XML;
+		san->sessionType = SML_SESSION_TYPE_CLIENT;
+		san->target = smlLocationNew("/", NULL, error);
+		if (!san->target)
+			goto error;
+
+		/* assemble a SyncML 1.1 notification */
+		char *xmlData = NULL;
+		unsigned int xmlSize = 0;
+		if (!smlNotificationAssemble11(san, &xmlData, &xmlSize, SML_VERSION_12, error))
+		{
+			smlNotificationFree(san);
+			goto error;
+		}
+		smlTrace(TRACE_INTERNAL,
+			"%s - converted OMA DS 1.2 SAN to SyncML 1.1 server alerted sync - %s",
+			__func__, VA_STRING(xmlData));
+
+		/* cleanup notification */
+		smlNotificationFree(san);
+		san = NULL;
+
+		/* run the parser on the converted SAN */
+		if (!parser->functions.start(parser->parser_userdata, xmlData, xmlSize, error)) {
+			smlSafeCFree(&xmlData);
+			goto error;
+		}
+		smlSafeCFree(&xmlData);
+	} else {
+		/* run the parser directly on the data */
+		if (!parser->functions.start(parser->parser_userdata, data, size, error))
+			goto error;
+	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -155,6 +216,7 @@ error:
 SmlBool smlParserGetHeader(SmlParser *parser, SmlHeader **header, SmlCred **cred, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, header, cred, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(header);
 	smlAssert(cred);
@@ -188,6 +250,7 @@ error:
 SmlParserResult smlParserGetCommand(SmlParser *parser, SmlCommand **cmd, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, cmd, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(cmd);
 	smlAssert(parser->functions.get_cmd);
@@ -221,6 +284,7 @@ error:
 SmlBool smlParserGetStatus(SmlParser *parser, SmlStatus **status, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, status, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(status);
 	smlAssert(parser->functions.get_status);
@@ -250,13 +314,14 @@ error:
 SmlBool smlParserEnd(SmlParser *parser, SmlBool *final, SmlBool *end, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, parser, final, end, error);
+	CHECK_ERROR_REF
 	smlAssert(parser);
 	smlAssert(parser->functions.end);
 	
 	if (!parser->functions.end(parser->parser_userdata, final, end, error))
 		goto error;
 	
-	smlTrace(TRACE_INTERNAL, "Final %i, End %i", final ? *final : -1, end ? *end : -1);
+	smlTrace(TRACE_INTERNAL, "%s: Final %i, End %i", __func__, final ? *final : -1, end ? *end : -1);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -269,8 +334,8 @@ error:
 /*@}*/
 
 /**
- * @defgroup Assembler SyncML Assembler
- * @ingroup PublicAPI
+ * @defgroup SmlAssembler SyncML Assembler
+ * @ingroup PublicLowLevelAPI
  * @brief Interfaces to assemble syncml messages
  * 
  */
@@ -287,17 +352,16 @@ error:
 SmlAssembler *smlAssemblerNew(SmlMimeType type, unsigned int limit, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%i, %i, %p)", __func__, type, limit, error);
+	CHECK_ERROR_REF
 	
 	SmlAssembler *assm = smlTryMalloc0(sizeof(SmlAssembler), error);
 	if (!assm)
 		goto error;
 	
 	assm->type = type;
-	assm->limit = limit;
-	assm->req_limit = 0;
+	assm->remoteMaxMsgSize = limit;
 	assm->empty = TRUE;
-	assm->requestedMaxObjSize = -1;
-	assm->sendingMaxObjSize = -1;
+	assm->remoteMaxObjSize = 0;
 	
 	switch (type) {
 		case SML_MIMETYPE_XML:
@@ -324,7 +388,7 @@ SmlAssembler *smlAssemblerNew(SmlMimeType type, unsigned int limit, SmlError **e
 	return assm;
 	
 error_free_assm:
-	g_free(assm);
+	smlSafeFree((gpointer *)&assm);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
@@ -345,7 +409,7 @@ void smlAssemblerFree(SmlAssembler *assm)
 	
 	g_hash_table_destroy(assm->options);
 	
-	g_free(assm);
+	smlSafeFree((gpointer *)&assm);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -364,6 +428,7 @@ void smlAssemblerFree(SmlAssembler *assm)
 SmlBool smlAssemblerStart(SmlAssembler *assm, SmlSession *session, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, session, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(session);
 	smlAssert(assm->functions.start);
@@ -391,12 +456,13 @@ error:
 SmlBool smlAssemblerRun(SmlAssembler *assm, char **data, unsigned int *size, SmlBool *end, SmlBool final, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %i, %p)", __func__, assm, data, size, end, final, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(data);
 	smlAssert(size);
 	smlAssert(assm->functions.run);
 	
-	if (!assm->functions.run(assm->assm_userdata, data, size, end, final, smlAssemblerGetLimit(assm), error))
+	if (!assm->functions.run(assm->assm_userdata, data, size, end, final, smlAssemblerGetRemoteMaxMsgSize(assm), error))
 		goto error;
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -419,6 +485,7 @@ error:
 unsigned int smlAssemblerCheckSize(SmlAssembler *assm, SmlBool headeronly, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, assm, headeronly, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(assm->functions.check_size);
 	
@@ -457,6 +524,18 @@ unsigned int smlAssemblerFlush(SmlAssembler *assm)
 	return ret;
 }
 
+void smlAssemblerRestoreCommands(SmlAssembler *assm)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, assm);
+	smlAssert(assm);
+	smlAssert(assm->functions.restore_cmds);
+	
+	assm->functions.restore_cmds(assm->assm_userdata);
+	assm->empty = FALSE;
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+}
+
 /** @brief Gets the available space in the assembler
  * 
  * This command tries to add the given command to the assembler without the
@@ -466,6 +545,7 @@ unsigned int smlAssemblerFlush(SmlAssembler *assm)
 SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent, SmlCommand *cmd, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %p)", __func__, assm, space, parent, cmd, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(space);
 	smlAssert(cmd);
@@ -474,7 +554,7 @@ SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent,
 	smlAssert(cmd->type == SML_COMMAND_TYPE_ADD || cmd->type == SML_COMMAND_TYPE_REPLACE);
 	unsigned int parentID = 0;
 	
-	int limit = smlAssemblerGetLimit(assm);
+	int limit = smlAssemblerGetRemoteMaxMsgSize(assm);
 	if (!limit) {
 		*space = -1;
 		smlTrace(TRACE_EXIT, "%s: No limit", __func__);
@@ -490,11 +570,22 @@ SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent,
 		parentID = parent->cmdID;
 	}
 	
-	/* Now we remove the item */
-	cmd->private.change.item->disabled = TRUE;
+	/* disable the items */
+	if (cmd->private.change.items)
+	{
+		guint i;
+		for (i=0; i < g_list_length(cmd->private.change.items); i++)
+		{
+			SmlItem *item = g_list_nth_data(cmd->private.change.items, i);
+			item->disabled = TRUE;
+		}
+	}
+
 	SmlBool noCmdID = FALSE;
 	if (!cmd->cmdID) {
 		noCmdID = TRUE;
+		/* FIXME: This code does not support more
+		 * FIXME: than 99.999 commands in one session. */
 		cmd->cmdID = 10000;
 	}
 	
@@ -502,7 +593,7 @@ SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent,
 		goto error_enable_item;
 	
 	/* Now check the size */
-	unsigned int size = 0;
+	int size = 0;
 	if (!(size = smlAssemblerCheckSize(assm, FALSE, error)))
 		goto error_remove;
 	
@@ -515,7 +606,16 @@ SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent,
 	if (!assm->functions.rem_cmd(assm->assm_userdata, parentID, error))
 		 goto error_remove;
 	
-	cmd->private.change.item->disabled = FALSE;
+	/* enable the items again */
+	if (cmd->private.change.items)
+	{
+		guint i;
+		for (i=0; i < g_list_length(cmd->private.change.items); i++)
+		{
+			SmlItem *item = g_list_nth_data(cmd->private.change.items, i);
+			item->disabled = FALSE;
+		}
+	}
 	
 	if (noCmdID)
 		cmd->cmdID = 0;
@@ -526,7 +626,15 @@ SmlBool smlAssemblerGetSpace(SmlAssembler *assm, int *space, SmlCommand *parent,
 error_remove:
 	assm->functions.rem_cmd(assm->assm_userdata, parentID, NULL);
 error_enable_item:
-	cmd->private.change.item->disabled = FALSE;
+	if (cmd->private.change.items)
+	{
+		guint i;
+		for (i=0; i < g_list_length(cmd->private.change.items); i++)
+		{
+			SmlItem *item = g_list_nth_data(cmd->private.change.items, i);
+			item->disabled = FALSE;
+		}
+	}
 	if (noCmdID)
 		cmd->cmdID = 0;
 error:
@@ -551,6 +659,7 @@ error:
 SmlAssemblerResult smlAssemblerStartCommand(SmlAssembler *assm, SmlCommand *parent, SmlCommand *cmd, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, assm, parent, cmd, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(cmd);
 	smlAssert(assm->functions.start_cmd);
@@ -570,10 +679,10 @@ SmlAssemblerResult smlAssemblerStartCommand(SmlAssembler *assm, SmlCommand *pare
 		goto error;
 	
 	/* Lets see if the new buffer is small enough */
-	int limit = smlAssemblerGetLimit(assm);
+	int limit = smlAssemblerGetRemoteMaxMsgSize(assm);
 	if (limit) {
 		/* Now check the size */
-		unsigned int size = 0;
+		int size = 0;
 		if (!(size = smlAssemblerCheckSize(assm, FALSE, error)))
 			goto error;
 		
@@ -585,7 +694,7 @@ SmlAssemblerResult smlAssemblerStartCommand(SmlAssembler *assm, SmlCommand *pare
 			smlTrace(TRACE_EXIT, "%s: Mismatch", __func__);
 			return SML_ASSEMBLER_RESULT_MISMATCH;
 		}
-		smlTrace(TRACE_INTERNAL, "size %i, limit %i", size, limit);
+		smlTrace(TRACE_INTERNAL, "%s: size %i, limit %i", __func__, size, limit);
 	}
 	
 	if (cmd->cmdID)
@@ -613,6 +722,7 @@ error:
 SmlBool smlAssemblerEndCommand(SmlAssembler *assm, SmlCommand *parent, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, parent, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(assm->functions.end_cmd);
 	
@@ -652,6 +762,7 @@ error:
 SmlBool smlAssemblerAddHeader(SmlAssembler *assm, SmlSession *session, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, session, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(session);
 	smlAssert(assm->functions.add_header);
@@ -666,7 +777,7 @@ SmlBool smlAssemblerAddHeader(SmlAssembler *assm, SmlSession *session, SmlError
 		goto error;
 	
 	/* Lets see if the new buffer is small enough */
-	if (smlAssemblerGetLimit(assm) && size > smlAssemblerGetLimit(assm)) {
+	if (smlAssemblerGetRemoteMaxMsgSize(assm) && size > smlAssemblerGetRemoteMaxMsgSize(assm)) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Limit to small. Unable to fit a the header");
 		goto error;
 	}
@@ -695,6 +806,7 @@ error:
 SmlAssemblerResult smlAssemblerReserveStatus(SmlAssembler *assm, unsigned int cmdRef, unsigned int msgRef, unsigned int cmdID, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %i, %i, %p)", __func__, assm, cmdRef, msgRef, cmdID, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(assm->functions.reserve_status);
 	
@@ -727,6 +839,7 @@ error:
 SmlAssemblerResult smlAssemblerAddStatusFull(SmlAssembler *assm, SmlStatus *status, SmlBool force, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, assm, status, force, error);
+	CHECK_ERROR_REF
 	smlAssert(assm);
 	smlAssert(status);
 	smlAssert(assm->functions.add_status);
@@ -738,10 +851,10 @@ SmlAssemblerResult smlAssemblerAddStatusFull(SmlAssembler *assm, SmlStatus *stat
 	
 	if (!force) {
 		/* Lets see if the new buffer is small enough */
-		int limit = smlAssemblerGetLimit(assm);
+		int limit = smlAssemblerGetRemoteMaxMsgSize(assm);
 		if (limit) {
 			/* Now check the size */
-			unsigned int size = 0;
+			int size = 0;
 			if (!(size = smlAssemblerCheckSize(assm, FALSE, error)))
 				goto error;
 			
@@ -769,6 +882,7 @@ error:
 
 SmlAssemblerResult smlAssemblerAddStatus(SmlAssembler *assm, SmlStatus *status, SmlError **error)
 {
+	CHECK_ERROR_REF
 	return smlAssemblerAddStatusFull(assm, status, FALSE, error);
 }
 
@@ -781,7 +895,7 @@ SmlAssemblerResult smlAssemblerAddStatus(SmlAssembler *assm, SmlStatus *status,
  */
 void smlAssemblerSetOption(SmlAssembler *assm, const char *optionname, const char *value)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %s, %s)", __func__, assm, optionname, value);
+	smlTrace(TRACE_ENTRY, "%s(%p, %s, %s)", __func__, assm, VA_STRING(optionname), VA_STRING(value));
 	smlAssert(assm);
 	smlAssert(optionname);
 	
@@ -799,109 +913,125 @@ void smlAssemblerSetOption(SmlAssembler *assm, const char *optionname, const cha
  */
 const char *smlAssemblerGetOption(SmlAssembler *assm, const char *optionname)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %s)", __func__, assm, optionname);
+	smlTrace(TRACE_ENTRY, "%s(%p, %s)", __func__, assm, VA_STRING(optionname));
 	smlAssert(assm);
 	smlAssert(optionname);
 	
 	const char *ret = g_hash_table_lookup(assm->options, optionname);
 	
-	smlTrace(TRACE_EXIT, "%s: %s", __func__, ret);
+	smlTrace(TRACE_EXIT, "%s: %s", __func__, VA_STRING(ret));
 	return ret;
 }
 
-/* Returns the lower of the 2 limits. There are 2 limits since one
- * limit might be requested by the other side, while the other limit might
- * be set by the local user to limit the size of the output buffer. This function
- * returns the lower one of the 2 */
+/* FIXME: DEPRECATED */
 unsigned int smlAssemblerGetLimit(SmlAssembler *assm)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, assm);
-	smlAssert(assm);
-	unsigned int ret = 0;
-	
-	if (assm->limit) {
-		if (assm->req_limit) {
-			ret = assm->limit < assm->req_limit ? assm->limit : assm->req_limit;
-		} else
-			ret = assm->limit;
-	} else
-		ret = assm->req_limit;
-	
-	smlTrace(TRACE_EXIT, "%s: %u", __func__, ret);
-	return ret;
+	return smlAssemblerGetRemoteMaxMsgSize(assm);
 }
 
+unsigned int smlAssemblerGetRemoteMaxMsgSize(SmlAssembler *assm)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, assm);
+	smlAssert(assm);
+
+	smlTrace(TRACE_EXIT, "%s: %i", __func__, assm->remoteMaxMsgSize);
+	return assm->remoteMaxMsgSize;
+}
+
+/* FIXME: DEPRECATED */
 void smlAssemblerSetRequestedLimit(SmlAssembler *assm, unsigned int limit)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, assm, limit);
-	smlAssert(assm);
-	
-	/* Dont update the limit if it is 0. A client might send a maxmsg size
-	 * in the first package, and none in the second. Then, the maxmsgsize from
-	 * the first package has to be used. See Syncml 1.1 MetInf Section 5.8 */
-	if (limit == 0) {
-		smlTrace(TRACE_EXIT, "%s: Not setting limit. limit is 0", __func__);
-		return;
-	}
-	assm->req_limit = limit;
-	
-	smlTrace(TRACE_EXIT, "%s", __func__);
+	smlAssemblerSetRemoteMaxMsgSize(assm, limit);
 }
 
+/* FIXME: DEPRECATED */
 void smlAssemblerSetLimit(SmlAssembler *assm, unsigned int limit)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, assm, limit);
-	smlAssert(assm);
-	
-	assm->limit = limit;
-	
-	smlTrace(TRACE_EXIT, "%s", __func__);
+	smlAssemblerSetRemoteMaxMsgSize(assm, limit);
 }
 
-/* Returns the lower of the 2 limits. There are 2 limits since one
- * limit might be requested by the other side, while the other limit might
- * be set by the local user to limit the size of the output buffer. This function
- * returns the lower one of the 2 */
+unsigned int smlAssemblerSetRemoteMaxMsgSize(SmlAssembler *assm, unsigned int limit)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %i)", __func__, assm, limit);
+	smlAssert(assm);
+
+	if (limit == 0)
+	{
+		/* Reset MaxMsgSize to unused state */
+		assm->remoteMaxMsgSize = limit;
+	} else {
+		if (limit < assm->remoteMaxMsgSize || assm->remoteMaxMsgSize == 0)
+		{
+			/* There is a real new limit. */
+			assm->remoteMaxMsgSize = limit;
+		} else {
+			smlTrace(TRACE_INTERNAL,
+				"%s: using old limit (%d) because of large new limit (%d).",
+				__func__, assm->remoteMaxMsgSize, limit);
+		}
+	}
+	
+	smlTrace(TRACE_EXIT, "%s - %d", __func__, assm->remoteMaxMsgSize);
+	return assm->remoteMaxMsgSize;
+}
+
+/* FIXME: DEPRECATED */
 int smlAssemblerGetSendingMaxObjSize(SmlAssembler *assm)
+{
+	return smlAssemblerGetRemoteMaxObjSize(assm);
+}
+
+/* FIXME: DEPRECATED */
+int smlAssemblerGetRequestedMaxObjSize(SmlAssembler *assm)
+{
+	return smlAssemblerGetRemoteMaxObjSize(assm);
+}
+
+
+unsigned int smlAssemblerGetRemoteMaxObjSize(SmlAssembler *assm)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, assm);
 	smlAssert(assm);
-	int ret = 0;
-	
-	/* If both limits are set, we return the lower limit. Otherwise we return the larger one */
-	if (assm->sendingMaxObjSize > 0 && assm->requestedMaxObjSize > 0)
-		ret = assm->sendingMaxObjSize < assm->requestedMaxObjSize ? assm->sendingMaxObjSize : assm->requestedMaxObjSize;
-	else
-		ret = assm->sendingMaxObjSize > assm->requestedMaxObjSize ? assm->sendingMaxObjSize : assm->requestedMaxObjSize;
-	
-	smlTrace(TRACE_EXIT, "%s: %i", __func__, ret);
-	return ret;
+
+	smlTrace(TRACE_EXIT, "%s: %i", __func__, assm->remoteMaxObjSize);
+	return assm->remoteMaxObjSize;
 }
 
+/* FIXME: DEPRECATED */
 void smlAssemblerSetRequestedMaxObjSize(SmlAssembler *assm, int limit)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %i)", __func__, assm, limit);
-	smlAssert(assm);
-	
-	assm->requestedMaxObjSize = limit;
-	
-	smlTrace(TRACE_EXIT, "%s", __func__);
-}
-
-int smlAssemblerGetRequestedMaxObjSize(SmlAssembler *assm)
-{
-	smlAssert(assm);
-	return assm->requestedMaxObjSize;
+	smlAssemblerSetRemoteMaxObjSize(assm, limit);
 }
 
+/* FIXME: DEPRECATED */
 void smlAssemblerSetSendingMaxObjSize(SmlAssembler *assm, int limit)
+{
+	smlAssemblerSetRemoteMaxObjSize(assm, limit);
+}
+
+unsigned int smlAssemblerSetRemoteMaxObjSize(SmlAssembler *assm, unsigned int limit)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i)", __func__, assm, limit);
 	smlAssert(assm);
+
+	if (limit == 0)
+	{
+		/* Reset MaxObjSize to unused state */
+		assm->remoteMaxObjSize = limit;
+	} else {
+		if (limit < assm->remoteMaxObjSize || assm->remoteMaxObjSize == 0)
+		{
+			/* There is a real new limit. */
+			assm->remoteMaxObjSize = limit;
+		} else {
+			smlTrace(TRACE_INTERNAL,
+				"%s: using old limit (%d) because of large new limit (%d).",
+				__func__, assm->remoteMaxObjSize, limit);
+		}
+	}
 	
-	assm->sendingMaxObjSize = limit;
-	
-	smlTrace(TRACE_EXIT, "%s", __func__);
+	smlTrace(TRACE_EXIT, "%s - %d", __func__, assm->remoteMaxObjSize);
+	return assm->remoteMaxObjSize;
 }
 
 SmlBool smlAssemblerIsEmpty(SmlAssembler *assm)
