@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,19 +27,18 @@ struct SmlParser {
 	void *parser_userdata;
 	SmlMimeType type;
 	unsigned int limit;
+	SmlManager *manager;
 };
 
 struct SmlAssembler {
 	SmlAssemblerFunctions functions;
 	void *assm_userdata;
 	SmlMimeType type;
-	unsigned int limit;
-	unsigned int req_limit;
 	GHashTable *options;
 	SmlBool empty;
-	
-	int sendingMaxObjSize;
-	int requestedMaxObjSize;
+
+	unsigned int remoteMaxMsgSize;
+	unsigned int remoteMaxObjSize;
 };
 
 #endif //_SML_PARSE_INTERNALS_H_
