@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,12 +29,15 @@ typedef enum SmlSessionType {
 
 typedef enum SmlSessionEventType {
 	SML_SESSION_EVENT_ERROR,
-	SML_SESSION_EVENT_COMMAND,
+	SML_SESSION_EVENT_COMMAND_START,
 	SML_SESSION_EVENT_CHILD_COMMAND,
 	SML_SESSION_EVENT_HEADER_REPLY,
 	SML_SESSION_EVENT_FINAL,
 	SML_SESSION_EVENT_END,
-	SML_SESSION_EVENT_FLUSH
+	SML_SESSION_EVENT_FLUSH,
+	SML_SESSION_EVENT_COMMAND_END,
+	SML_SESSION_EVENT_RESPONSE_URI,
+	SML_SESSION_EVENT_ESTABLISHED,
 } SmlSessionEventType;
 
 typedef void (* SmlSessionEventCallback) (SmlSession *session, SmlSessionEventType type, SmlCommand *command, SmlCommand *parent, SmlStatus *reply, SmlError *error, void *userdata);
@@ -54,15 +58,15 @@ void smlSessionUseOnlyReplace(SmlSession *session, SmlBool onlyReplace);
 void smlSessionUseNumberOfChanges(SmlSession *session, SmlBool support);
 void smlSessionUseLargeObjects(SmlSession *session, SmlBool support);
 
-void smlSessionSetSendingMaxMsgSize(SmlSession *session, unsigned int size);
-void smlSessionSetReceivingMaxMsgSize(SmlSession *session, unsigned int size);
-unsigned int smlSessionGetSendingMaxMsgSize(SmlSession *session);
+void smlSessionSetRemoteMaxMsgSize(SmlSession *session, unsigned int size);
+void smlSessionSetLocalMaxMsgSize(SmlSession *session, unsigned int size);
+unsigned int smlSessionGetRemoteMaxMsgSize(SmlSession *session);
+unsigned int smlSessionGetLocalMaxMsgSize(SmlSession *session);
 
-void smlSessionSetSendingMaxObjSize(SmlSession *session, int limit);
-void smlSessionSetRequestedMaxObjSize(SmlSession *session, int limit);
-int smlSessionGetSendingMaxObjSize(SmlSession *session);
-void smlSessionSetReceivingMaxObjSize(SmlSession *session, unsigned int limit);
-unsigned int smlSessionGetReceivingMaxObjSize(SmlSession *session);
+void smlSessionSetLocalMaxObjSize(SmlSession *session, unsigned int limit);
+void smlSessionSetRemoteMaxObjSize(SmlSession *session, unsigned int limit);
+unsigned int smlSessionGetLocalMaxObjSize(SmlSession *session);
+unsigned int smlSessionGetRemoteMaxObjSize(SmlSession *session);
 
 const char *smlSessionGetSessionID(SmlSession *session);
 void smlSessionSetSessionID(SmlSession *session, const char *sessionID);
@@ -70,8 +74,10 @@ SmlProtocolVersion smlSessionGetVersion(SmlSession *session);
 void smlSessionDispatchEvent(SmlSession *session, SmlSessionEventType type, SmlCommand *cmd, SmlCommand *parent, SmlStatus *headerreply, SmlError *error);
 SmlLocation *smlSessionGetSource(SmlSession *session);
 SmlLocation *smlSessionGetTarget(SmlSession *session);
-void smlSessionSetCred(SmlSession *session, SmlCred *cred);
-SmlCred *smlSessionGetCred(SmlSession *session);
+/* This function MUST NOT be used for servers.
+ * This function is only used to set the credentials for a client.
+ */
+void smlSessionRegisterCred(SmlSession *session, SmlCred *cred);
 
 /* Session Data input */
 SmlBool smlSessionReceiveHeader(SmlSession *session, SmlHeader *header, SmlError **error);
@@ -91,4 +97,16 @@ void smlSessionSetSendingLimit(SmlSession *session, unsigned int limit) LIBSYNCM
 void smlSessionSetReceivingLimit(SmlSession *session, unsigned int limit) LIBSYNCML_DEPRECATED;
 unsigned int smlSessionGetSendingLimit(SmlSession *session) LIBSYNCML_DEPRECATED;
 
+/* expire date: 20090626 */
+void smlSessionSetSendingMaxMsgSize(SmlSession *session, unsigned int size) LIBSYNCML_DEPRECATED;
+void smlSessionSetReceivingMaxMsgSize(SmlSession *session, unsigned int size) LIBSYNCML_DEPRECATED;
+unsigned int smlSessionGetSendingMaxMsgSize(SmlSession *session) LIBSYNCML_DEPRECATED;
+
+/* expire date: 20090626 */
+void smlSessionSetSendingMaxObjSize(SmlSession *session, int limit) LIBSYNCML_DEPRECATED;
+void smlSessionSetRequestedMaxObjSize(SmlSession *session, int limit) LIBSYNCML_DEPRECATED;
+int smlSessionGetSendingMaxObjSize(SmlSession *session) LIBSYNCML_DEPRECATED;
+void smlSessionSetReceivingMaxObjSize(SmlSession *session, unsigned int limit) LIBSYNCML_DEPRECATED;
+unsigned int smlSessionGetReceivingMaxObjSize(SmlSession *session) LIBSYNCML_DEPRECATED;
+
 #endif //_SML_SESSION_H_
