@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,13 +41,18 @@ struct SmlSession {
 	char *sessionID;
 	
 	SmlLocation *target;
+	SmlLocation *orgTarget;
 	SmlLocation *source;
+	char *responseURI;
+	SmlChal *chal;
 	SmlCred *cred;
-	
-	unsigned int incomingMaxMsgSize;
-	/** This is the maximum object size we want to receive from the other side.
-	 * This value will be transmitted in the <MaxObjSize> tag */
-	unsigned int receivingMaxObjSize;
+
+	/* These are the local limits of MaxMsgSize and MaxObjSize.
+	 * These are the limits which are send to the remote peer
+	 * but they are not the remote limits.
+	 */
+	unsigned int localMaxMsgSize;
+	unsigned int localMaxObjSize;
 	
 	GList *pendingReplies;
 	
@@ -56,6 +62,8 @@ struct SmlSession {
 	
 	GList *commands;
 	SmlBool final;
+	SmlBool authenticate;
+	SmlBool established;
 	
 	SmlMimeType type;
 	
@@ -64,15 +72,17 @@ struct SmlSession {
 	SmlBool sending;
 	GList *commandStack;
 	SmlBool hasCommand;
+	SmlBool assmHasHeader;
 	
 	SmlBool end;
-	SmlBool want_end;
+	GMutex *reportEnd;
+	SmlBool reportedEnd;
 	
 	SmlCommand *parentCommand;
 	
-	/* The buffer that holds the incoming item for
-	 * large objects */
-	SmlItem *incomingBuffer;
+	/* The buffer that holds the incoming command which has
+	 * an item with a large object. */
+	SmlCommand *incomingBuffer;
 	
 	/* The outgoing command that gets fragmented */
 	SmlCommand *frag_command;
@@ -93,4 +103,14 @@ struct SmlPendingStatus {
 	void *userdata;
 };
 
+SmlBool smlSessionTryLock(SmlSession *session);
+void smlSessionLock(SmlSession *session);
+void smlSessionUnlock(SmlSession *session);
+
+void smlSessionRestoreTargetURI(SmlSession *session);
+SmlBool smlSessionSetResponseURI(
+		SmlSession *session,
+		const char *responseURI,
+		SmlError **error);
+
 #endif //_SML_SESSION_INTERNALS_H_
