@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,15 +24,35 @@
 
 #include "sml_queue_internals.h"
 
-typedef void *(* SmlTransportInitializeFn) (SmlTransport *tsp, const void *data, SmlError **error);
+typedef SmlBool (* SmlTransportSetConfigOptionFn) (SmlTransport *tsp, const char *name, const char *value, SmlError **error);
+typedef SmlBool (* SmlTransportSetConnectionTypeFn) (SmlTransport *tsp, SmlTransportConnectionType type, SmlError **error);
+typedef SmlBool (* SmlTransportInitializeFn) (SmlTransport *tsp, SmlError **error);
+
+/* This function is used for transport clients
+ * to support redirects from the server.
+ * Example: Funambol works with such redirects
+ *          because of Tomcat's session management.
+ */
+typedef SmlBool (* SmlTransportSetResponseUriFn) (SmlTransport *tsp, const char *uri, SmlError **error);
+
+/* This function is used to add some kind of session id
+ * to the URL of the server. This is necessary to
+ * avoid session hijacking.
+ */
+typedef char * (* SmlTransportGetResponseUriFn) (SmlLink *link, SmlSession *session, SmlError **error);
+
 typedef SmlBool (* SmlTransportFinalizeFn) (void *data, SmlError **error);
 
 typedef void (* SmlTransportConnectFn) (void *data);
-typedef void (* SmlTransportDisconnectFn) (void *data, void *link);
-typedef void (* SmlTransportSendFn) (void *userdata, void *link, SmlTransportData *data, SmlError *error);
+typedef void (* SmlTransportDisconnectFn) (void *data, void *link_data);
+typedef void (* SmlTransportSendFn) (void *userdata, void *link_data, SmlTransportData *data, SmlError *error);
 
 typedef struct SmlTransportFunctions {
+	SmlTransportSetConfigOptionFn set_config_option;
+	SmlTransportSetConnectionTypeFn set_connection_type;
 	SmlTransportInitializeFn initialize;
+	SmlTransportSetResponseUriFn set_response_uri;
+	SmlTransportGetResponseUriFn get_response_uri;
 	SmlTransportFinalizeFn finalize;
 	SmlTransportConnectFn connect;
 	SmlTransportDisconnectFn disconnect;
@@ -58,13 +79,17 @@ struct SmlTransport {
 	SmlQueue *command_queue;
 	SmlTransportEventCb event_callback;
 	void *event_callback_userdata;
+	gint event_callback_ref_count;
 	SmlError *cached_error;
 	
-	GList *links;
+	SmlBool connected;
+	GHashTable *links;
+	GMutex *links_mutex;
+	unsigned int connections;
+	GMutex *connections_mutex;
 };
 
 struct SmlLink {
-	SmlQueue *event_queue;
 	SmlTransport *tsp;
 	void *link_data;
 	gint32 refCount;
@@ -110,4 +135,7 @@ struct SmlTransportData {
 void smlTransportWorkerHandler(void *message, void *userdata);
 SmlBool smlTransportReceiveEvent(SmlTransport *tsp, SmlLink *link, SmlTransportEventType type, SmlTransportData *data, SmlError *error);
 
+SmlBool smlTransportSetResponseURI(SmlTransport *tsp, const char *uri, SmlError **error);
+char *smlTransportGetResponseURI(SmlLink *link, SmlSession *session, SmlError **error);
+
 #endif //_SML_TRANSPORT_INTERNALS_H_
