@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +19,26 @@
  *
  */
 
+/**
+ * @defgroup PublicAPI libsyncml API
+ * @ingroup PublicAPI
+ * @brief The public part
+ * 
+ */
+/**
+ * @defgroup PublicObjectAPI libsyncml Object API
+ * @ingroup PublicAPI
+ * @brief The public part
+ * 
+ */
+/**
+ * @defgroup PublicLowLevelAPI Low Level API
+ * @ingroup PublicAPI
+ * @brief The public part
+ * 
+ */
+/*@{*/
+
 #ifndef _SYNCML_H
 #define _SYNCML_H
 
@@ -84,8 +105,35 @@ typedef int SmlBool;
 #include <libsyncml/sml_base64.h>
 #include <libsyncml/sml_md5.h>
 
+const char *smlGetLibraryVersion();
+const char *smlGetLibrarySoName();
+
 #ifdef __cplusplus
 }
 #endif
 
 #endif
+
+/*@}*/
+
+/** @mainpage
+ *
+ * This library implements the following protocols:
+ *
+ *   - SyncML 1.0 Data Synchronization Protocol
+ *   - SyncML 1.1 Data Synchronization Protocol
+ *   - OMA DS 1.1
+ *   - OMA DS 1.2
+ *
+ * The following transports are supported:
+ *
+ *   - HTTP client
+ *   - HTTP server
+ *   - OBEX client
+ *   - OBEX server
+ *
+ * SANs for HTTP clients are not supported because HTTP OTA requires
+ * direct GSM network access.
+ *
+ * XML and WBXML are both supported.
+ */
