@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007-2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,22 +22,36 @@
 #ifndef _HTTP_CLIENT_INTERNALS_H_
 #define _HTTP_CLIENT_INTERNALS_H_
 
+#ifdef HAVE_LIBSOUP22_SOLARIS
+#include <libsoup/soup-session-sync.h>
+#else
 #include <libsoup/soup-session-async.h>
+#endif
+
 #include <libsoup/soup-uri.h>
+#ifndef HAVE_LIBSOUP22
+#include <libsoup/soup-auth.h>
+#endif
 
 typedef struct SmlTransportHttpClientEnv {
+#ifdef HAVE_LIBSOUP22
 	SoupUri *uri;
+#else
+	SoupURI *uri;
+#endif
 	SoupSession *session;
 	SmlTransport *tsp;
 	uint16_t port;
 	char *url;
 	char *proxy;
-	SmlTransportData *data;
 	char *username;
 	char *password;
+	char *cafile;
+
 	SmlBool connectDone;
-	GMutex *cleanupMutex;
-	SmlBool shutdown;
+	SmlBool disconnectDone;
 } SmlTransportHttpClientEnv;
 
+SmlBool smlTransportHttpClientNew(SmlTransport *tsp, SmlError **error);
+
 #endif //_HTTP_CLIENT_INTERNALS_H_
