@@ -1,11 +1,12 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; version 
- * 2.1 of the License.
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,16 +22,23 @@
 #ifndef _OBEX_SERVER_INTERNALS_H_
 #define _OBEX_SERVER_INTERNALS_H_
 
+#include "obex_bluetooth.h"
 #include <openobex/obex.h>
 
 #define STREAM_CHUNK 4096
 
 typedef struct SmlTransportObexServerEnv {
-	SmlTransportObexServerType type;
+	SmlTransportConnectionType type;
 	SmlTransport *tsp;
 	char *path;
+	uint16_t port;
+#ifdef ENABLE_BLUETOOTH
+	bdaddr_t *bdaddr;
+#endif
+	char *irda_service;
 	
-	int fd;
+	obex_t *handle;
+
 	GSource *source;
 	GSourceFuncs *functions;
 	
@@ -43,7 +51,6 @@ typedef struct SmlLinkObexServerEnv {
 	uint32_t conid;
 	obex_t *handle;
 	uint8_t *stream_chunk;
-	int fd;
 	SmlBool destroy;
 	SmlBool disconnect;
 	GSource *source;
@@ -51,4 +58,6 @@ typedef struct SmlLinkObexServerEnv {
 	SmlTransportData *send_data;
 } SmlLinkObexServerEnv;
 
+SmlBool smlTransportObexServerNew(SmlTransport *tsp, SmlError **error);
+
 #endif //_OBEX_SERVER_INTERNALS_H_
