@@ -1,6 +1,7 @@
 /*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +26,6 @@
 #include <config.h>
 
 #include <libsyncml/sml_support.h>
-#include <libsyncml/transports/obex_client.h>
 
 #include <libsyncml/objects/sml_auth.h>
 #include <libsyncml/objects/sml_devinf_obj.h>
@@ -51,6 +51,9 @@ GList *dbSlow = NULL;
 GList *dbTypes = NULL;
 
 SmlBool onlyInfo = FALSE;
+char *useTimeAnchor = NULL;
+unsigned int recvLimit = 65535;
+unsigned int maxObjSize = 3000000;
 
 static void usage (char *name, int ecode)
 {
@@ -75,16 +78,18 @@ static void usage (char *name, int ecode)
 	fprintf (stderr, "\t\t\t\t\tnotes and \"text/x-vMessage\" for SMS\n");
 	fprintf (stderr, "\t\t\t\tPath to the file to add. The file has to be a VCard, VCalendar, etc.\n\n");
 	fprintf (stderr, "[--wbxml]\t\t\tUse wbxml (WAP Binary XML) instead of plain xml\n\n");
-	fprintf (stderr, "[--recvLimit <limit>]\t\t\tLimit the size of the receiving buffer to this size (Needed for some phones)\n\n");
-	fprintf (stderr, "[--maxObjSize <limit>]\t\t\tThe maximum size of a object that we can receive (Needed for some phones)\n\n");
+	fprintf (stderr, "[--recvLimit <limit>]\t\t\tLimit the size of the receiving buffer to this size (default: %d)\n\n", recvLimit);
+	fprintf (stderr, "[--maxObjSize <limit>]\t\t\tThe maximum size of a object that we can receive (default: %d)\n\n", maxObjSize);
 	fprintf (stderr, "[--useStringTable]\t\t\tUse wbxml string tables (Improves transmission size, but not supported by some phones)\n\n");
+	fprintf (stderr, "[--disableNumberOfChanges]\t\t\tDisable NumberOfChanges feature (the most new phones need it) (default: enabled)\n\n");
+	fprintf (stderr, "[--useTimeAnchor]\t\t\tUse timestamps as anchors (only for new Nokia series 40 phones)\n\n");
 	fprintf (stderr, "[--dumpinfo]\t\t\tPrint info about the phone at the end which can be sent to the developers\n");
 	exit (ecode);
 }
 
 static SmlBool _recv_change(SmlDsSession *dsession, SmlChangeType type, const char *uid, char *data, unsigned int size, const char *contenttype, void *userdata, SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %p, %i, %s, %p, %p)", __func__, dsession, type, uid, data, size, contenttype, userdata, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %p, %i, %s, %p, %p)", __func__, dsession, type, VA_STRING(uid), data, size, VA_STRING(contenttype), userdata, error);
 	
 	printf("Received a ");
 	
@@ -122,7 +127,7 @@ static void _recv_sync_reply(SmlSession *session, SmlStatus *status, void *userd
 
 static void _add_reply(SmlDsSession *session, SmlStatus *status, const char *newuid, void *userdata)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, session, status, newuid, userdata);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, session, status, VA_STRING(newuid), userdata);
 	
 	printf("Received an reply to our Add command: %i. The new uid is %s\n", smlStatusGetCode(status), newuid);
 
@@ -135,6 +140,7 @@ GList *paths = NULL;
 static void _recv_sync(SmlDsSession *dsession, unsigned int numchanges, void *userdata)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, dsession, numchanges, userdata);
+	SmlError *error = NULL;
 	
 	printf("Going to receive %i changes\n", numchanges);
 	
@@ -147,7 +153,8 @@ static void _recv_sync(SmlDsSession *dsession, unsigned int numchanges, void *us
 	}
 	
 	if (!onlyInfo) {
-		smlDsSessionSendSync(dsession, num, _recv_sync_reply, NULL, NULL);
+		if (!smlDsSessionSendSync(dsession, num, _recv_sync_reply, NULL, &error))
+			goto error;
 		
 		t = types;
 		GList *p = paths;
@@ -181,10 +188,14 @@ static void _recv_sync(SmlDsSession *dsession, unsigned int numchanges, void *us
 			p = p->next;
 		}
 		
-		smlDsSessionCloseSync(dsession, NULL);
+		if (!smlDsSessionCloseSync(dsession, &error))
+			goto error;
 	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s - %s", __func__, smlErrorPrint(&error));
+	smlErrorDeref(&error);
 }
 
 static void _recv_alert_reply(SmlSession *session, SmlStatus *status, void *userdata)
@@ -198,8 +209,11 @@ static void _recv_alert_reply(SmlSession *session, SmlStatus *status, void *user
 
 static SmlBool _recv_alert(SmlDsSession *dsession, SmlAlertType type, const char *last, const char *next, void *userdata)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %s, %p)", __func__, dsession, type, last, next, userdata);
+	smlTrace(TRACE_ENTRY, "%s(%p, %i, %s, %s, %p)", __func__, dsession, type, VA_STRING(last), VA_STRING(next), userdata);
 	SmlError *error = NULL;
+	const char *anchor = next;
+	if (useTimeAnchor)
+		anchor = useTimeAnchor;
 	
 	printf("Received an Alert for the DS Server at %s: Type: %i, Last %s, Next %s\n", smlDsSessionGetLocation(dsession), type, last, next);
 	
@@ -215,15 +229,27 @@ static SmlBool _recv_alert(SmlDsSession *dsession, SmlAlertType type, const char
 	if (onlyInfo) {
 		if (!smlDsSessionSendAlert(dsession, SML_ALERT_ONE_WAY_FROM_SERVER_BY_SERVER, last, next, _recv_alert_reply, NULL, &error))
 			goto error;
-	} else if (slow) {
-		printf("Slowsyncing\n");
-		if (!smlDsSessionSendAlert(dsession, SML_ALERT_SLOW_SYNC, last, next, _recv_alert_reply, NULL, &error))
+	} else if (slow && type != SML_ALERT_SLOW_SYNC) {
+		/* SLOW-SYNC enforced by tool */
+		printf("Enforcing SLOW-SYNC\n");
+		if (!smlDsSessionSendAlert(dsession, SML_ALERT_SLOW_SYNC, NULL, anchor, _recv_alert_reply, NULL, &error))
 			goto error;
-			
+
 		smlTrace(TRACE_EXIT, "%s: Slow syncing", __func__);
 		return FALSE;
 	} else {
-		if (!smlDsSessionSendAlert(dsession, SML_ALERT_TWO_WAY_BY_SERVER, last, next, _recv_alert_reply, NULL, &error))
+		/* We only support SLOW-SYNC and TWO-WAY-SYNC */
+		/* FIXME: Why do we use alert 206? */
+		/* FIXME: Usually 200 and 201 are the correct alert codes. */
+		/* FIXME: 206 is only useful in SANs */
+		if (type != SML_ALERT_SLOW_SYNC &&
+		    type != SML_ALERT_TWO_WAY &&
+		    type != SML_ALERT_TWO_WAY_BY_SERVER)
+		{
+			smlErrorSet(&error, SML_ERROR_GENERIC, "Unsupported alert type %d.", type);
+			goto error;
+		}
+		if (!smlDsSessionSendAlert(dsession, type, last, anchor, _recv_alert_reply, NULL, &error))
 			goto error;
 	}
 	
@@ -252,12 +278,13 @@ static void _ds_alert(SmlDsSession *dsession, void *userdata)
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-unsigned int recvLimit = 0;
-unsigned int maxObjSize = 0;
 SmlBool dumpinfo = FALSE;
 SmlBool useStringTable = FALSE;
+SmlBool useNumberOfChanges = TRUE;
+SmlBool useLargeObjs = TRUE;
 SmlProtocolVersion sessionVersion = SML_VERSION_UNKNOWN;
 SmlDevInf *deviceDevinf = NULL;
+SmlDevInf *devinf = NULL;
 
 static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSession *session, SmlError *error, void *userdata)
 {
@@ -268,6 +295,9 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 		case SML_MANAGER_CONNECT_DONE:
 			printf("connection with device succeeded\n");
 			break;
+		case SML_MANAGER_SESSION_ESTABLISHED:
+			printf("session incl. authentication successfully established\n");
+			break;
 		case SML_MANAGER_DISCONNECT_DONE:
 			printf("connection with device has ended\n");
 			break;
@@ -279,19 +309,44 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 		case SML_MANAGER_SESSION_NEW:
 			printf("Just received a new session with ID %s\n", smlSessionGetSessionID(session));
 			sessionVersion = smlSessionGetVersion(session);
-			
+
+			if (smlSessionGetRemoteMaxMsgSize(session) > recvLimit &&
+			    recvLimit > 0)
+				recvLimit = smlSessionGetRemoteMaxMsgSize(session);
+
 			if (recvLimit)
-				smlSessionSetReceivingMaxMsgSize(session, recvLimit);
+				smlSessionSetLocalMaxMsgSize(session, recvLimit);
 				
+			if (smlSessionGetRemoteMaxObjSize(session) > maxObjSize &&
+			    maxObjSize > 0)
+				maxObjSize = smlSessionGetRemoteMaxObjSize(session);
+
 			if (maxObjSize)
-				smlSessionSetReceivingMaxObjSize(session, maxObjSize);
+				smlSessionSetLocalMaxObjSize(session, maxObjSize);
 				
 			if (useStringTable)
 				smlSessionUseStringTable(session, TRUE);
+				
+			smlSessionUseNumberOfChanges(session, useNumberOfChanges);
+			smlSessionUseLargeObjects(session, useLargeObjs);
+
 			break;
 		case SML_MANAGER_SESSION_FINAL:
-			if (!deviceDevinf && (deviceDevinf = smlDevInfAgentGetDevInf(agent)))
+			if (!deviceDevinf && (deviceDevinf = smlDevInfAgentGetDevInf(agent))) {
 				printf("Received the DevInf\n");
+				smlSessionUseNumberOfChanges(session, smlDevInfSupportsNumberOfChanges(deviceDevinf));
+				smlDevInfSetSupportsNumberOfChanges(devinf, smlDevInfSupportsNumberOfChanges(deviceDevinf));
+			}
+			if ((deviceDevinf && smlDevInfSupportsLargeObjs(deviceDevinf)) ||
+			    (!deviceDevinf && useLargeObjs)) {
+				/* enable standard compliant large object support */
+				if (recvLimit)
+					smlSessionSetLocalMaxMsgSize(session, recvLimit);
+				if (maxObjSize)
+					smlSessionSetLocalMaxObjSize(session, maxObjSize);
+				smlSessionUseLargeObjects(session, TRUE);
+				smlDevInfSetSupportsLargeObjs(devinf, TRUE);
+			}
 			
 			/* We want to know about the device so we check if we received
 			 * the devinf already */
@@ -302,11 +357,12 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			}
 			
 			printf("Session %s reported final. flushing\n", smlSessionGetSessionID(session));
-			
+
+			SmlError *locerror = NULL;
 			if (onlyInfo && deviceDevinf)
-				smlSessionEnd(session, NULL);
+				smlSessionEnd(session, &locerror);
 			else
-				smlSessionFlush(session, TRUE, NULL);
+				smlSessionFlush(session, TRUE, &locerror);
 			break;
 		case SML_MANAGER_SESSION_END:
 			printf("Session %s has ended\n", smlSessionGetSessionID(session));
@@ -430,9 +486,9 @@ GMainContext *_sessions_attach()
 
 int main(int argc, char *argv[])
 {
-	SmlTransportObexClientConfig config;
-	config.type = SML_OBEX_TYPE_USB;
-	config.url = NULL;
+	SmlTransportConnectionType type = SML_TRANSPORT_CONNECTION_TYPE_USB;
+	char *url = NULL;
+	char *port = NULL;
 	
 	SmlMimeType mimeType = SML_MIMETYPE_XML;
 	
@@ -467,37 +523,37 @@ int main(int argc, char *argv[])
 			}
 			
 			errno = 0;
-			config.port = strtol(argv[i], (char **)NULL, 10);
-			if (errno)
+			port = argv[i];
+			if (!port)
 				usage (argv[0], 1);
 		} else if (!strcmp (arg, "-b")) {
 #ifdef ENABLE_BLUETOOTH
-			config.type = SML_OBEX_TYPE_BLUETOOTH;
+			type = SML_TRANSPORT_CONNECTION_TYPE_BLUETOOTH;
 
 			i++;
 			if (!argv[i])
 				usage (argv[0], 1);
-			config.url = g_strdup(argv[i]);
+			url = g_strdup(argv[i]);
 
 			i++;
 			if (!argv[i])
 				usage (argv[0], 1);
 			errno = 0;
-			config.port = strtol(argv[i], (char **)NULL, 10);
-			if (errno)
+			port = argv[i];
+			if (!port)
 				usage (argv[0], 1);
 #else
 			printf("Bluetooth not available in this build\n");
 			return 1;
 #endif
 		} else if (!strcmp (arg, "-s")) {
-			config.type = SML_OBEX_TYPE_SERIAL;
+			type = SML_TRANSPORT_CONNECTION_TYPE_SERIAL;
 
 			i++;
 			if (!argv[i])
 				usage (argv[0], 1);
 
-			config.url = g_strdup(argv[i]);
+			url = g_strdup(argv[i]);
 
  		} else if (!strcmp (arg, "--identifier")) {
 			i++;
@@ -526,6 +582,12 @@ int main(int argc, char *argv[])
 			maxObjSize = atoi(argv[i]);
 		} else if (!strcmp (arg, "--useStringTable")) {
 			useStringTable = TRUE;
+		} else if (!strcmp (arg, "--disableNumberOfChanges")) {
+			useNumberOfChanges = FALSE;
+		} else if (!strcmp (arg, "--useTimeAnchor")) {
+			useTimeAnchor = malloc(sizeof(char)*17);
+			time_t htime = time(NULL);
+			strftime(useTimeAnchor, 17, "%Y%m%dT%H%M%SZ", gmtime(&htime));
 		} else if (!strcmp (arg, "--version")) {
 			i++;
 			if (!argv[i])
@@ -566,6 +628,14 @@ int main(int argc, char *argv[])
 	if (!manager)
 		goto error;
 	smlManagerSetEventCallback(manager, _manager_event, NULL);
+	if (useLargeObjs)
+	{
+		/* This is a server which supports SyncML 1.1 or higher.
+		 * So large objects must be supported.
+		 */
+		smlManagerSetLocalMaxMsgSize(manager, recvLimit);
+		smlManagerSetLocalMaxObjSize(manager, maxObjSize);
+	}
 	
 	/* The authenticator */
 	SmlAuthenticator *auth = smlAuthNew(&error);
@@ -580,9 +650,12 @@ int main(int argc, char *argv[])
 	
 	
 	/* Now create the devinf handler */
-	SmlDevInf *devinf = smlDevInfNew("LibSyncmML", SML_DEVINF_DEVTYPE_WORKSTATION, &error);
+	devinf = smlDevInfNew("LibSyncmML", SML_DEVINF_DEVTYPE_WORKSTATION, &error);
 	if (!devinf)
 		goto error_free_manager;
+	smlDevInfSetSupportsNumberOfChanges(devinf, useNumberOfChanges);
+	smlDevInfSetSupportsLargeObjs(devinf, useLargeObjs);
+	smlDevInfSetSupportsUTC(devinf, TRUE);
 	
 	agent = smlDevInfAgentNew(devinf, &error);
 	if (!agent)
@@ -598,6 +671,7 @@ int main(int argc, char *argv[])
 	SmlNotification *san = smlNotificationNew(sanVersion, SML_SAN_UIMODE_UNSPECIFIED, SML_SAN_INITIATOR_USER, 1, identifier, "/", mimeType, &error);
 	if (!san)
 		goto error;
+	smlNotificationSetManager(san, manager);
 		
 	for (i = 1; i < argc; i++) {
 		char *arg = argv[i];
@@ -608,6 +682,10 @@ int main(int argc, char *argv[])
 				usage (argv[0], 1);
 			
 			dbTypes = g_list_append(dbTypes, argv[i]);
+			if (!strstr(argv[i], "/"))
+				fprintf(stderr, "WARNING: Specified database type \"%s\" doesn't look like a valid MIME type!\n"
+						"WARNING: (Mixed up database path/location with database type?)\n", argv[i] ? argv[i] : "");
+
 			
 			i++;
 			if (!argv[i])
@@ -647,8 +725,8 @@ int main(int argc, char *argv[])
 				smlDevInfDataStoreSetRxPref(datastore, SML_ELEMENT_TEXT_VCARD, "2.1");
 				smlDevInfDataStoreSetTxPref(datastore, SML_ELEMENT_TEXT_VCARD, "2.1");
 			} else if (!strcmp(g_list_last(dbTypes)->data, SML_ELEMENT_TEXT_VCAL)) {
-				smlDevInfDataStoreSetRxPref(datastore, SML_ELEMENT_TEXT_VCAL, "2.0");
-				smlDevInfDataStoreSetTxPref(datastore, SML_ELEMENT_TEXT_VCAL, "2.0");
+				smlDevInfDataStoreSetRxPref(datastore, SML_ELEMENT_TEXT_VCAL, "1.0");
+				smlDevInfDataStoreSetTxPref(datastore, SML_ELEMENT_TEXT_VCAL, "1.0");
 			} else if (!strcmp(g_list_last(dbTypes)->data, SML_ELEMENT_TEXT_PLAIN)) {
 				smlDevInfDataStoreSetRxPref(datastore, SML_ELEMENT_TEXT_PLAIN, "1.0");
 				smlDevInfDataStoreSetTxPref(datastore, SML_ELEMENT_TEXT_PLAIN, "1.0");
@@ -681,7 +759,10 @@ int main(int argc, char *argv[])
 	}
 
 	/* Initialize the Transport */
-	if (!smlTransportInitialize(client, &config, &error))
+	if (!smlTransportSetConfigOption(client, "PORT", port, &error) ||
+	    !smlTransportSetConfigOption(client, "URL", url, &error) ||
+	    !smlTransportSetConnectionType(client, type, &error) ||
+	    !smlTransportInitialize(client, &error))
 		goto error;
 
 	/* Run the manager */
@@ -690,7 +771,7 @@ int main(int argc, char *argv[])
 	
 	if (!smlTransportConnect(client, &error))
 		goto error;
-	
+
 	if (!smlNotificationSend(san, client, &error))
 		goto error;
 	
@@ -733,16 +814,16 @@ int main(int argc, char *argv[])
 				printf("DB Type: %s\n", (char *)g_list_nth_data(dbTypes, i));
 				printf("DB Slow: %i\n\n", GPOINTER_TO_INT(g_list_nth_data(dbSlow, i)));
 			}
-			printf("Bluetooth: %s\n", config.type == SML_OBEX_TYPE_BLUETOOTH ? "Yes" : "Unknown"); 
+			printf("Bluetooth: %s\n", type == SML_TRANSPORT_CONNECTION_TYPE_BLUETOOTH ? "Yes" : "Unknown"); 
 			printf("Wbxml: %s\n", mimeType == SML_MIMETYPE_WBXML ? "Yes" : "No");
 			printf("SyncML Version: 1.%i\n", sessionVersion - 1);
-			printf("SupportsNumberofChanges: %s\n", smlDevInfSupportsNumberOfChanges(deviceDevinf) ? "Yes" : "No");
+			printf("SupportsNumberOfChanges: %s\n", smlDevInfSupportsNumberOfChanges(deviceDevinf) ? "Yes" : "No");
 			printf("SupportsLargeObjects: %s\n", smlDevInfSupportsLargeObjs(deviceDevinf) ? "Yes" : "No");
 		}
 	}
 	
 	g_free(identifier);
-	g_free(config.url);
+	g_free(url);
 	
 	return 0;
 
